@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 set -e
 
@@ -9,11 +9,14 @@ usage() {
 	Check out and install a meson project
 
 	Options:
-	  -Dkey=val      Option to pass on to meson
-	  --subdir       Build subdirectory instead of whole project
-	  --prepare      Script to run before build
+	  -Dkey=val          Option to pass on to meson
+	  --subdir=DIR       Build subdirectory instead of whole project
+	  --prepare=SCRIPT   Script to run before build
+	  --libdir=DIR       Setup the project with a different libdir
+	  --destdir=DIR      Install the project to DIR, can be used
+	                     several times to install to multiple destdirs
 
-	  -h, --help     Display this help
+	  -h, --help         Display this help
 
 	EOF
 }
@@ -23,6 +26,8 @@ TEMP=$(getopt \
   --options='D:h' \
   --longoptions='subdir:' \
   --longoptions='prepare:' \
+  --longoptions='libdir:' \
+  --longoptions='destdir:' \
   --longoptions='help' \
   -- "$@")
 
@@ -32,6 +37,7 @@ unset TEMP
 MESON_OPTIONS=()
 SUBDIR=.
 PREPARE=:
+DESTDIRS=()
 
 while true; do
   case "$1" in
@@ -50,6 +56,16 @@ while true; do
       shift 2
     ;;
 
+    --libdir)
+      MESON_OPTIONS+=( --libdir=$2 )
+      shift 2
+    ;;
+
+    --destdir)
+      DESTDIRS+=( $2 )
+      shift 2
+    ;;
+
     -h|--help)
       usage
       exit 0
@@ -70,6 +86,8 @@ fi
 REPO_URL="$1"
 COMMIT="$2"
 
+[[ ${#DESTDIRS[@]} == 0 ]] && DESTDIRS+=( / )
+
 CHECKOUT_DIR=$(mktemp --directory)
 trap "rm -rf $CHECKOUT_DIR" EXIT
 
@@ -78,5 +96,12 @@ git clone --depth 1 "$REPO_URL" -b "$COMMIT" "$CHECKOUT_DIR"
 pushd "$CHECKOUT_DIR/$SUBDIR"
 sh -c "$PREPARE"
 meson setup --prefix=/usr _build "${MESON_OPTIONS[@]}"
-meson install -C _build
+
+# Install it to all specified dest dirs
+for destdir in "${DESTDIRS[@]}"; do
+    # don't use --destdir when installing to root,
+    # so post-install hooks are run
+    [[ $destdir == / ]] && destdir=
+    sudo meson install -C _build ${destdir:+--destdir=$destdir}
+done
 popd
