@@ -21,8 +21,7 @@
  * Author: Carlos Garnacho <carlosg@gnome.org>
  */
 
-#ifndef CLUTTER_GRAB_H
-#define CLUTTER_GRAB_H
+#pragma once
 
 #if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
 #error "Only <clutter/clutter.h> can be included directly."
@@ -30,11 +29,15 @@
 
 #include <glib-object.h>
 
+#include "clutter-macros.h"
+#include "clutter-enums.h"
+
 #define CLUTTER_TYPE_GRAB (clutter_grab_get_type ())
-typedef struct _ClutterGrab ClutterGrab;
+CLUTTER_EXPORT
+G_DECLARE_FINAL_TYPE (ClutterGrab, clutter_grab, CLUTTER, GRAB, GObject)
 
 CLUTTER_EXPORT
-GType clutter_grab_get_type (void) G_GNUC_CONST;
+void clutter_grab_activate (ClutterGrab *grab);
 
 CLUTTER_EXPORT
 void clutter_grab_dismiss (ClutterGrab *grab);
@@ -43,9 +46,4 @@ CLUTTER_EXPORT
 ClutterGrabState clutter_grab_get_seat_state (ClutterGrab *grab);
 
 CLUTTER_EXPORT
-ClutterGrab * clutter_grab_ref (ClutterGrab *grab);
-
-CLUTTER_EXPORT
-void clutter_grab_unref (ClutterGrab *grab);
-
-#endif /* CLUTTER_GRAB_H */
+gboolean clutter_grab_is_revoked (ClutterGrab *grab);
