@@ -27,12 +27,9 @@
 
 #include "clutter/clutter-paint-node-private.h"
 
-#include <pango/pango.h>
-
 #include "cogl/cogl.h"
 #include "clutter/clutter-actor-private.h"
 #include "clutter/clutter-blur-private.h"
-#include "clutter/clutter-color.h"
 #include "clutter/clutter-debug.h"
 #include "clutter/clutter-private.h"
 #include "clutter/clutter-paint-context-private.h"
@@ -86,6 +83,8 @@ struct _ClutterRootNode
 {
   ClutterPaintNode parent_instance;
 
+  ClutterColorState *color_state;
+
   CoglFramebuffer *framebuffer;
 
   CoglBufferBit clear_flags;
@@ -100,6 +99,7 @@ clutter_root_node_pre_draw (ClutterPaintNode    *node,
 {
   ClutterRootNode *rnode = (ClutterRootNode *) node;
 
+  clutter_paint_context_push_color_state (paint_context, rnode->color_state);
   clutter_paint_context_push_framebuffer (paint_context, rnode->framebuffer);
 
   cogl_framebuffer_clear (rnode->framebuffer,
@@ -113,6 +113,7 @@ static void
 clutter_root_node_post_draw (ClutterPaintNode    *node,
                              ClutterPaintContext *paint_context)
 {
+  clutter_paint_context_pop_color_state (paint_context);
   clutter_paint_context_pop_framebuffer (paint_context);
 }
 
@@ -121,7 +122,8 @@ clutter_root_node_finalize (ClutterPaintNode *node)
 {
   ClutterRootNode *rnode = (ClutterRootNode *) node;
 
-  g_object_unref (rnode->framebuffer);
+  g_clear_object (&rnode->color_state);
+  g_clear_object (&rnode->framebuffer);
 
   CLUTTER_PAINT_NODE_CLASS (clutter_root_node_parent_class)->finalize (node);
 }
@@ -151,25 +153,24 @@ clutter_root_node_init (ClutterRootNode *self)
 }
 
 ClutterPaintNode *
-clutter_root_node_new (CoglFramebuffer    *framebuffer,
-                       const ClutterColor *clear_color,
-                       CoglBufferBit       clear_flags)
+clutter_root_node_new (CoglFramebuffer   *framebuffer,
+                       ClutterColorState *color_state,
+                       const CoglColor   *clear_color,
+                       CoglBufferBit      clear_flags)
 {
   ClutterRootNode *res;
 
   g_return_val_if_fail (framebuffer, NULL);
+  g_return_val_if_fail (CLUTTER_IS_COLOR_STATE (color_state), NULL);
 
   res = _clutter_paint_node_create (CLUTTER_TYPE_ROOT_NODE);
 
-  cogl_color_init_from_4f (&res->clear_color,
-                           clear_color->red / 255.0,
-                           clear_color->green / 255.0,
-                           clear_color->blue / 255.0,
-                           clear_color->alpha / 255.0);
+  res->clear_color = *clear_color;
   cogl_color_premultiply (&res->clear_color);
 
   res->framebuffer = g_object_ref (framebuffer);
   res->clear_flags = clear_flags;
+  res->color_state = g_object_ref (color_state);
 
   return (ClutterPaintNode *) res;
 }
@@ -403,6 +404,23 @@ clutter_pipeline_node_draw (ClutterPaintNode    *node,
   if (node->operations == NULL)
     return;
 
+  if (!cogl_pipeline_has_capability (pnode->pipeline,
+                                     CLUTTER_PIPELINE_CAPABILITY,
+                                     CLUTTER_PIPELINE_CAPABILITY_COLOR_STATE))
+    {
+      ClutterColorState *color_state =
+        clutter_paint_context_get_color_state (paint_context);
+      ClutterColorState *target_color_state =
+        clutter_paint_context_get_target_color_state (paint_context);
+
+      clutter_color_state_add_pipeline_transform (color_state,
+                                                  target_color_state,
+                                                  pnode->pipeline);
+    }
+
+  if (!cogl_pipeline_get_name (pnode->pipeline))
+    cogl_pipeline_set_static_name (pnode->pipeline, node->name);
+
   fb = clutter_paint_context_get_framebuffer (paint_context);
 
   for (i = 0; i < node->operations->len; i++)
@@ -502,7 +520,7 @@ clutter_pipeline_node_new (CoglPipeline *pipeline)
   res = _clutter_paint_node_create (CLUTTER_TYPE_PIPELINE_NODE);
 
   if (pipeline != NULL)
-    res->pipeline = g_object_ref (pipeline);
+    res->pipeline = cogl_pipeline_copy (pipeline);
 
   return (ClutterPaintNode *) res;
 }
@@ -555,7 +573,7 @@ clutter_color_node_init (ClutterColorNode *cnode)
  *   clutter_paint_node_unref() when done
  */
 ClutterPaintNode *
-clutter_color_node_new (const ClutterColor *color)
+clutter_color_node_new (const CoglColor *color)
 {
   ClutterPipelineNode *cnode;
 
@@ -563,16 +581,11 @@ clutter_color_node_new (const ClutterColor *color)
 
   if (color != NULL)
     {
-      CoglColor cogl_color;
+      CoglColor pipeline_color;
 
-      cogl_color_init_from_4f (&cogl_color,
-                               color->red / 255.0,
-                               color->green / 255.0,
-                               color->blue / 255.0,
-                               color->alpha / 255.0);
-      cogl_color_premultiply (&cogl_color);
-
-      cogl_pipeline_set_color (cnode->pipeline, &cogl_color);
+      pipeline_color = *color;
+      cogl_color_premultiply (&pipeline_color);
+      cogl_pipeline_set_color (cnode->pipeline, &pipeline_color);
     }
 
   return (ClutterPaintNode *) cnode;
@@ -636,7 +649,7 @@ clutter_scaling_filter_to_cogl_pipeline_filter (ClutterScalingFilter filter)
 /**
  * clutter_texture_node_new:
  * @texture: a #CoglTexture
- * @color: (allow-none): a #ClutterColor used for blending, or %NULL
+ * @color: (allow-none): a #CoglColor used for blending, or %NULL
  * @min_filter: the minification filter for the texture
  * @mag_filter: the magnification filter for the texture
  *
@@ -645,7 +658,7 @@ clutter_scaling_filter_to_cogl_pipeline_filter (ClutterScalingFilter filter)
  * This function will take a reference on @texture, so it is safe to
  * call g_object_unref() on @texture when it returns.
  *
- * The @color must not be pre-multiplied with its #ClutterColor.alpha
+ * The @color must not be pre-multiplied with its #CoglColor.alpha
  * channel value; if @color is %NULL, a fully opaque white color will
  * be used for blending.
  *
@@ -654,12 +667,12 @@ clutter_scaling_filter_to_cogl_pipeline_filter (ClutterScalingFilter filter)
  */
 ClutterPaintNode *
 clutter_texture_node_new (CoglTexture          *texture,
-                          const ClutterColor   *color,
+                          const CoglColor      *color,
                           ClutterScalingFilter  min_filter,
                           ClutterScalingFilter  mag_filter)
 {
   ClutterPipelineNode *tnode;
-  CoglColor cogl_color;
+  CoglColor pipeline_color;
   CoglPipelineFilter min_f, mag_f;
 
   g_return_val_if_fail (COGL_IS_TEXTURE (texture), NULL);
@@ -674,185 +687,17 @@ clutter_texture_node_new (CoglTexture          *texture,
 
   if (color != NULL)
     {
-      cogl_color_init_from_4f (&cogl_color,
-                               color->red / 255.0,
-                               color->green / 255.0,
-                               color->blue / 255.0,
-                               color->alpha / 255.0);
-      cogl_color_premultiply (&cogl_color);
+      pipeline_color = *color;
+      cogl_color_premultiply (&pipeline_color);
     }
   else
-    cogl_color_init_from_4f (&cogl_color, 1.0, 1.0, 1.0, 1.0);
+    cogl_color_init_from_4f (&pipeline_color, 1.0, 1.0, 1.0, 1.0);
 
-  cogl_pipeline_set_color (tnode->pipeline, &cogl_color);
+  cogl_pipeline_set_color (tnode->pipeline, &pipeline_color);
 
   return (ClutterPaintNode *) tnode;
 }
 
-
-/**
- * ClutterTextNode:
- */
-struct _ClutterTextNode
-{
-  ClutterPaintNode parent_instance;
-
-  PangoLayout *layout;
-  CoglColor color;
-};
-
-/**
- * ClutterTextNodeClass:
- *
- * The `ClutterTextNodeClass` structure is an opaque
- * type whose contents cannot be directly accessed.
- */
-struct _ClutterTextNodeClass
-{
-  ClutterPaintNodeClass parent_class;
-};
-
-G_DEFINE_TYPE (ClutterTextNode, clutter_text_node, CLUTTER_TYPE_PAINT_NODE)
-
-static void
-clutter_text_node_finalize (ClutterPaintNode *node)
-{
-  ClutterTextNode *tnode = CLUTTER_TEXT_NODE (node);
-
-  g_clear_object (&tnode->layout);
-
-  CLUTTER_PAINT_NODE_CLASS (clutter_text_node_parent_class)->finalize (node);
-}
-
-static gboolean
-clutter_text_node_pre_draw (ClutterPaintNode    *node,
-                            ClutterPaintContext *paint_context)
-{
-  ClutterTextNode *tnode = CLUTTER_TEXT_NODE (node);
-
-  return tnode->layout != NULL;
-}
-
-static void
-clutter_text_node_draw (ClutterPaintNode    *node,
-                        ClutterPaintContext *paint_context)
-{
-  ClutterTextNode *tnode = CLUTTER_TEXT_NODE (node);
-  PangoRectangle extents;
-  CoglFramebuffer *fb;
-  guint i;
-
-  if (node->operations == NULL)
-    return;
-
-  fb = get_target_framebuffer (node, paint_context);
-
-  pango_layout_get_pixel_extents (tnode->layout, NULL, &extents);
-
-  for (i = 0; i < node->operations->len; i++)
-    {
-      const ClutterPaintOperation *op;
-      float op_width, op_height;
-      gboolean clipped = FALSE;
-
-      op = &g_array_index (node->operations, ClutterPaintOperation, i);
-
-      switch (op->opcode)
-        {
-        case PAINT_OP_TEX_RECT:
-          op_width = op->op.texrect[2] - op->op.texrect[0];
-          op_height = op->op.texrect[3] - op->op.texrect[1];
-
-          /* if the primitive size was smaller than the layout,
-           * we clip the layout when drawin, to avoid spilling
-           * it out
-           */
-          if (extents.width > op_width ||
-              extents.height > op_height)
-            {
-              cogl_framebuffer_push_rectangle_clip (fb,
-                                                    op->op.texrect[0],
-                                                    op->op.texrect[1],
-                                                    op->op.texrect[2],
-                                                    op->op.texrect[3]);
-              clipped = TRUE;
-            }
-
-          cogl_pango_show_layout (fb,
-                                  tnode->layout,
-                                  op->op.texrect[0],
-                                  op->op.texrect[1],
-                                  &tnode->color);
-
-          if (clipped)
-            cogl_framebuffer_pop_clip (fb);
-          break;
-
-        case PAINT_OP_TEX_RECTS:
-        case PAINT_OP_MULTITEX_RECT:
-        case PAINT_OP_PRIMITIVE:
-        case PAINT_OP_INVALID:
-          break;
-        }
-    }
-}
-
-static void
-clutter_text_node_class_init (ClutterTextNodeClass *klass)
-{
-  ClutterPaintNodeClass *node_class = CLUTTER_PAINT_NODE_CLASS (klass);
-
-  node_class->pre_draw = clutter_text_node_pre_draw;
-  node_class->draw = clutter_text_node_draw;
-  node_class->finalize = clutter_text_node_finalize;
-}
-
-static void
-clutter_text_node_init (ClutterTextNode *self)
-{
-  cogl_color_init_from_4f (&self->color, 0.0, 0.0, 0.0, 1.0);
-}
-
-/**
- * clutter_text_node_new:
- * @layout: (allow-none): a #PangoLayout, or %NULL
- * @color: (allow-none): the color used to paint the layout,
- *   or %NULL
- *
- * Creates a new #ClutterPaintNode that will paint a #PangoLayout
- * with the given color.
- *
- * This function takes a reference on the passed @layout, so it
- * is safe to call g_object_unref() after it returns.
- *
- * Return value: (transfer full): the newly created #ClutterPaintNode.
- *   Use clutter_paint_node_unref() when done
- */
-ClutterPaintNode *
-clutter_text_node_new (PangoLayout        *layout,
-                       const ClutterColor *color)
-{
-  ClutterTextNode *res;
-
-  g_return_val_if_fail (layout == NULL || PANGO_IS_LAYOUT (layout), NULL);
-
-  res = _clutter_paint_node_create (CLUTTER_TYPE_TEXT_NODE);
-
-  if (layout != NULL)
-    res->layout = g_object_ref (layout);
-
-  if (color != NULL)
-    {
-      cogl_color_init_from_4f (&res->color,
-                               color->red / 255.0,
-                               color->green / 255.0,
-                               color->blue / 255.0,
-                               color->alpha / 255.0);
-    }
-
-  return (ClutterPaintNode *) res;
-}
-
 /**
  * ClutterClipNode:
  */
@@ -1003,6 +848,7 @@ clutter_actor_node_pre_draw (ClutterPaintNode    *node,
                              ClutterPaintContext *paint_context)
 {
   ClutterActorNode *actor_node = CLUTTER_ACTOR_NODE (node);
+  ClutterColorState *color_state;
 
   if (actor_node->opacity_override != -1)
     {
@@ -1014,6 +860,9 @@ clutter_actor_node_pre_draw (ClutterPaintNode    *node,
 
   CLUTTER_SET_PRIVATE_FLAGS (actor_node->actor, CLUTTER_IN_PAINT);
 
+  color_state = clutter_actor_get_color_state (actor_node->actor);
+  clutter_paint_context_push_color_state (paint_context, color_state);
+
   return TRUE;
 }
 
@@ -1032,6 +881,8 @@ clutter_actor_node_post_draw (ClutterPaintNode    *node,
 {
   ClutterActorNode *actor_node = CLUTTER_ACTOR_NODE (node);
 
+  clutter_paint_context_pop_color_state (paint_context);
+
   CLUTTER_UNSET_PRIVATE_FLAGS (actor_node->actor, CLUTTER_IN_PAINT);
 
  if (actor_node->opacity_override != -1)
@@ -1166,6 +1017,7 @@ clutter_layer_node_pre_draw (ClutterPaintNode *node,
                              ClutterPaintContext *paint_context)
 {
   ClutterLayerNode *lnode = (ClutterLayerNode *) node;
+  ClutterColorState *color_state;
 
   /* if we were unable to create an offscreen buffer for this node, then
    * we simply ignore it
@@ -1174,6 +1026,8 @@ clutter_layer_node_pre_draw (ClutterPaintNode *node,
     return FALSE;
 
   clutter_paint_context_push_framebuffer (paint_context, lnode->offscreen);
+  color_state = clutter_paint_context_get_color_state (paint_context);
+  clutter_paint_context_push_target_color_state (paint_context, color_state);
 
   /* clear out the target framebuffer */
   cogl_framebuffer_clear4f (lnode->offscreen,
@@ -1200,12 +1054,29 @@ clutter_layer_node_post_draw (ClutterPaintNode    *node,
   /* switch to the previous framebuffer */
   cogl_framebuffer_pop_matrix (lnode->offscreen);
   clutter_paint_context_pop_framebuffer (paint_context);
+  clutter_paint_context_pop_target_color_state (paint_context);
 
   if (!node->operations)
     return;
 
   fb = clutter_paint_context_get_framebuffer (paint_context);
 
+  if (!cogl_pipeline_has_capability (lnode->pipeline,
+                                     CLUTTER_PIPELINE_CAPABILITY,
+                                     CLUTTER_PIPELINE_CAPABILITY_COLOR_STATE))
+    {
+      ClutterColorState *color_state;
+      ClutterColorState *target_color_state;
+
+      color_state =
+        clutter_paint_context_get_color_state (paint_context);
+      target_color_state =
+        clutter_paint_context_get_target_color_state (paint_context);
+      clutter_color_state_add_pipeline_transform (color_state,
+                                                  target_color_state,
+                                                  lnode->pipeline);
+    }
+
   for (i = 0; i < node->operations->len; i++)
     {
       const ClutterPaintOperation *op;
@@ -1352,6 +1223,10 @@ clutter_blit_node_draw (ClutterPaintNode    *node,
   if (node->operations == NULL)
     return;
 
+  g_warn_if_fail (clutter_color_state_equals (
+      clutter_paint_context_get_color_state (paint_context),
+      clutter_paint_context_get_target_color_state (paint_context)));
+
   framebuffer = get_target_framebuffer (node, paint_context);
 
   for (i = 0; i < node->operations->len; i++)
@@ -1370,14 +1245,14 @@ clutter_blit_node_draw (ClutterPaintNode    *node,
           op_width = op->op.texrect[6] - op->op.texrect[4];
           op_height = op->op.texrect[7] - op->op.texrect[5];
 
-          cogl_blit_framebuffer (blit_node->src,
+          cogl_framebuffer_blit (blit_node->src,
                                  framebuffer,
-                                 op->op.texrect[0],
-                                 op->op.texrect[1],
-                                 op->op.texrect[4],
-                                 op->op.texrect[5],
-                                 op_width,
-                                 op_height,
+                                 (int) op->op.texrect[0],
+                                 (int) op->op.texrect[1],
+                                 (int) op->op.texrect[4],
+                                 (int) op->op.texrect[5],
+                                 (int) op_width,
+                                 (int) op_height,
                                  &error);
 
           if (error)
@@ -1457,7 +1332,7 @@ clutter_blit_node_new (CoglFramebuffer *src)
  * @height: Height of region to copy
  *
  * Adds a new blit rectangle to the stack of rectangles. All the
- * constraints of [func@Cogl.blit_framebuffer] apply here.
+ * constraints of [method@Cogl.Framebuffer.blit] apply here.
  */
 void
 clutter_blit_node_add_blit_rectangle (ClutterBlitNode *blit_node,
@@ -1492,7 +1367,6 @@ struct _ClutterBlurNode
   ClutterLayerNode parent_instance;
 
   ClutterBlur *blur;
-  unsigned int radius;
 };
 
 G_DEFINE_TYPE (ClutterBlurNode, clutter_blur_node, CLUTTER_TYPE_LAYER_NODE)
@@ -1555,24 +1429,26 @@ clutter_blur_node_new (unsigned int width,
                        float        radius)
 {
   g_autoptr (CoglOffscreen) offscreen = NULL;
+  g_autoptr (CoglTexture) texture = NULL;
   g_autoptr (GError) error = NULL;
   ClutterLayerNode *layer_node;
   ClutterBlurNode *blur_node;
-  CoglContext *context;
-  CoglTexture *texture;
+  ClutterContext *context;
+  ClutterBackend *backend;
+  CoglContext *cogl_context;
   ClutterBlur *blur;
 
   g_return_val_if_fail (radius >= 0.0, NULL);
 
+  context = _clutter_context_get_default ();
+  backend = clutter_context_get_backend (context);
+  cogl_context = clutter_backend_get_cogl_context (backend);
   blur_node = _clutter_paint_node_create (CLUTTER_TYPE_BLUR_NODE);
-  blur_node->radius = radius;
-  context = clutter_backend_get_cogl_context (clutter_get_default_backend ());
-  texture = cogl_texture_2d_new_with_size (context, width, height);
+  texture = cogl_texture_2d_new_with_size (cogl_context, width, height);
 
   cogl_texture_set_premultiplied (texture, TRUE);
 
   offscreen = cogl_offscreen_new_with_texture (texture);
-  g_object_unref (texture);
   if (!cogl_framebuffer_allocate (COGL_FRAMEBUFFER (offscreen), &error))
     {
       g_warning ("Unable to allocate paint node offscreen: %s",
