@@ -36,9 +36,8 @@
 
 #include <glib.h>
 
-#include "cogl/cogl-buffer.h"
+#include "cogl/cogl-buffer-impl-private.h"
 #include "cogl/cogl-context.h"
-#include "cogl/cogl-gl-header.h"
 
 G_BEGIN_DECLS
 
@@ -60,7 +59,8 @@ struct _CoglBuffer
 
   CoglBufferFlags flags;
 
-  GLuint gl_handle; /* OpenGL handle */
+  CoglBufferImpl *impl;
+
   unsigned int size; /* size of the buffer, in bytes */
   CoglBufferUpdateHint update_hint;
 
@@ -70,20 +70,7 @@ struct _CoglBuffer
 
   unsigned int store_created : 1;
 
-  void * (* map_range) (CoglBuffer       *buffer,
-                        size_t            offset,
-                        size_t            size,
-                        CoglBufferAccess  access,
-                        CoglBufferMapHint hints,
-                        GError          **error);
-
-  void (* unmap) (CoglBuffer *buffer);
-
-  gboolean (* set_data) (CoglBuffer  *buffer,
-                         unsigned int offset,
-                         const void  *data,
-                         unsigned int size,
-                         GError     **error);
+  unsigned int use_malloc: 1;
 };
 struct _CoglBufferClass
 {
