@@ -40,7 +40,7 @@
 #include "cogl/winsys/cogl-winsys-private.h"
 
 
-G_DEFINE_TYPE (CoglDisplay, cogl_display, G_TYPE_OBJECT);
+G_DEFINE_FINAL_TYPE (CoglDisplay, cogl_display, G_TYPE_OBJECT);
 
 static const CoglWinsysVtable *
 _cogl_display_get_winsys (CoglDisplay *display)
@@ -63,7 +63,6 @@ cogl_display_dispose (GObject *object)
     }
 
   g_clear_object (&display->renderer);
-  g_clear_object (&display->onscreen_template);
 
   G_OBJECT_CLASS (cogl_display_parent_class)->dispose (object);
 }
@@ -82,27 +81,16 @@ cogl_display_class_init (CoglDisplayClass *class)
 }
 
 CoglDisplay *
-cogl_display_new (CoglRenderer         *renderer,
-                  CoglOnscreenTemplate *onscreen_template)
+cogl_display_new (CoglRenderer *renderer)
 {
+  g_return_val_if_fail (renderer != NULL, NULL);
+
   CoglDisplay *display = g_object_new (COGL_TYPE_DISPLAY, NULL);
-  GError *error = NULL;
-
-  _cogl_init ();
-
-  display->renderer = renderer;
-  if (renderer)
-    g_object_ref (renderer);
-  else
-    display->renderer = cogl_renderer_new ();
-
-  if (!cogl_renderer_connect (display->renderer, &error))
-    g_error ("Failed to connect to renderer: %s\n", error->message);
 
+  display->renderer = g_object_ref (renderer);
+  renderer->display = display;
   display->setup = FALSE;
 
-  cogl_display_set_onscreen_template (display, onscreen_template);
-
   return display;
 }
 
@@ -112,26 +100,6 @@ cogl_display_get_renderer (CoglDisplay *display)
   return display->renderer;
 }
 
-void
-cogl_display_set_onscreen_template (CoglDisplay *display,
-                                    CoglOnscreenTemplate *onscreen_template)
-{
-  g_return_if_fail (display->setup == FALSE);
-
-  if (onscreen_template)
-    g_object_ref (onscreen_template);
-
-  if (display->onscreen_template)
-    g_object_unref (display->onscreen_template);
-
-  display->onscreen_template = onscreen_template;
-
-  /* NB: we want to maintain the invariable that there is always an
-   * onscreen template associated with a CoglDisplay... */
-  if (!onscreen_template)
-    display->onscreen_template = cogl_onscreen_template_new (NULL);
-}
-
 gboolean
 cogl_display_setup (CoglDisplay *display,
                     GError **error)
