@@ -55,7 +55,6 @@ G_BEGIN_DECLS
 #define COGL_SUB_TEXTURE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  COGL_TYPE_SUB_TEXTURE, CoglSubTextureClass))
 
 typedef struct _CoglSubTextureClass CoglSubTextureClass;
-typedef struct _CoglSubTexture CoglSubTexture;
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (CoglSubTexture, g_object_unref)
 
@@ -77,13 +76,13 @@ GType               cogl_sub_texture_get_type       (void) G_GNUC_CONST;
  * Creates a high-level #CoglSubTexture representing a sub-region of
  * any other #CoglTexture. The sub-region must strictly lye within the
  * bounds of the @parent_texture. The returned texture implements the
- * #CoglMetaTexture interface because it's not a low level texture
+ * #CoglTexture interface because it's not a low level texture
  * that hardware can understand natively.
  *
  * Remember: Unless you are using high level drawing APIs such
  * as cogl_rectangle() or other APIs documented to understand the
- * #CoglMetaTexture interface then you need to use the
- * #CoglMetaTexture interface to resolve a #CoglSubTexture into a
+ * #CoglTexture interface then you need to use the
+ * #CoglTexture interface to resolve a #CoglSubTexture into a
  * low-level texture before drawing.
  *
  * Return value: (transfer full): A newly allocated #CoglSubTexture
