@@ -32,6 +32,7 @@
 
 #pragma once
 
+#include "cogl/cogl-driver-private.h"
 #include "cogl/cogl-types.h"
 #include "cogl/cogl-context.h"
 #include "cogl/cogl-texture.h"
@@ -75,36 +76,6 @@ _cogl_gl_error_to_string (GLenum error_code);
 
 #endif /* COGL_ENABLE_DEBUG */
 
-typedef struct _CoglGLContext {
-  GArray           *texture_units;
-  int               active_texture_unit;
-
-  /* This is used for generated fake unique sampler object numbers
-   when the sampler object extension is not supported */
-  GLuint next_fake_sampler_object_number;
-} CoglGLContext;
-
-CoglGLContext *
-_cogl_driver_gl_context (CoglContext *context);
-
-gboolean
-_cogl_driver_gl_context_init (CoglContext *context);
-
-void
-_cogl_driver_gl_context_deinit (CoglContext *context);
-
-void
-_cogl_driver_gl_flush_framebuffer_state (CoglContext          *context,
-                                         CoglFramebuffer      *draw_buffer,
-                                         CoglFramebuffer      *read_buffer,
-                                         CoglFramebufferState  state);
-
-CoglFramebufferDriver *
-_cogl_driver_gl_create_framebuffer_driver (CoglContext                        *context,
-                                           CoglFramebuffer                    *framebuffer,
-                                           const CoglFramebufferDriverConfig  *driver_config,
-                                           GError                            **error);
-
 GLenum
 _cogl_gl_util_get_error (CoglContext *ctx);
 
@@ -114,9 +85,6 @@ _cogl_gl_util_clear_gl_errors (CoglContext *ctx);
 gboolean
 _cogl_gl_util_catch_out_of_memory (CoglContext *ctx, GError **error);
 
-gboolean
-_cogl_driver_gl_is_hardware_accelerated (CoglContext *context);
-
 /*
  * _cogl_context_get_gl_extensions:
  * @context: A CoglContext
@@ -131,9 +99,6 @@ _cogl_context_get_gl_extensions (CoglContext *context);
 const char *
 _cogl_context_get_gl_version (CoglContext *context);
 
-const char *
-_cogl_context_get_gl_vendor (CoglContext *context);
-
 /* Parses a GL version number stored in a string. @version_string must
  * point to the beginning of the version number (ie, it can't point to
  * the "OpenGL ES" part on GLES). The version number can be followed
@@ -145,23 +110,6 @@ _cogl_gl_util_parse_gl_version (const char *version_string,
                                 int *major_out,
                                 int *minor_out);
 
-CoglGraphicsResetStatus
-_cogl_gl_get_graphics_reset_status (CoglContext *context);
-
-CoglTimestampQuery *
-cogl_gl_create_timestamp_query (CoglContext *context);
-
-void
-cogl_gl_free_timestamp_query (CoglContext        *context,
-                              CoglTimestampQuery *query);
-
-int64_t
-cogl_gl_timestamp_query_get_time_ns (CoglContext        *context,
-                                     CoglTimestampQuery *query);
-
-int64_t
-cogl_gl_get_gpu_time_ns (CoglContext *context);
-
 #ifndef GL_FRAMEBUFFER
 #define GL_FRAMEBUFFER		0x8D40
 #endif
