@@ -23,7 +23,6 @@
 
 #include "mtk/mtk-rectangle.h"
 
-
 MtkRectangle *
 mtk_rectangle_copy (const MtkRectangle *rect)
 {
@@ -152,7 +151,7 @@ mtk_rectangle_union (const MtkRectangle *rect1,
  * @src2: another #MtkRectangle
  * @dest: (out caller-allocates): an empty #MtkRectangle, to be filled
  *   with the coordinates of the intersection.
- * 
+ *
  * Find the intersection between the two rectangles
  *
  * Returns: TRUE is some intersection exists and is not degenerate, FALSE
@@ -285,6 +284,41 @@ mtk_rectangle_contains_rect (const MtkRectangle *outer_rect,
     inner_rect->y + inner_rect->height <= outer_rect->y + outer_rect->height;
 }
 
+/**
+ * mtk_rectangle_contains_point:
+ * @rect: A rectangle
+ * @x: X coordinate of the point
+ * @y: Y coordinate of the point
+ *
+ * Returns: Whether the rectangle contains the point
+ */
+gboolean
+mtk_rectangle_contains_point (const MtkRectangle *rect,
+                              int                 x,
+                              int                 y)
+{
+  return mtk_rectangle_contains_pointf (rect, x, y);
+}
+
+/**
+ * mtk_rectangle_contains_pointf:
+ * @rect: A rectangle
+ * @x: X coordinate of the point
+ * @y: Y coordinate of the point
+ *
+ * Returns: Whether the rectangle contains the point
+ */
+gboolean
+mtk_rectangle_contains_pointf (const MtkRectangle *rect,
+                               float               x,
+                               float               y)
+{
+  return (x >= rect->x &&
+          x <  (rect->x + rect->width) &&
+          y >= rect->y &&
+          y <  (rect->y + rect->height));
+}
+
 /**
  * mtk_rectangle_to_graphene_rect:
  * @rect: A rectangle
@@ -323,10 +357,10 @@ mtk_rectangle_from_graphene_rect (const graphene_rect_t *rect,
     case MTK_ROUNDING_STRATEGY_SHRINK:
       {
         *dest = (MtkRectangle) {
-          .x = ceilf (rect->origin.x),
-          .y = ceilf (rect->origin.y),
-          .width = floorf (rect->size.width),
-          .height = floorf (rect->size.height),
+          .x = (int) ceilf (rect->origin.x),
+          .y = (int) ceilf (rect->origin.y),
+          .width = (int) floorf (rect->size.width),
+          .height = (int) floorf (rect->size.height),
         };
       }
       break;
@@ -337,20 +371,20 @@ mtk_rectangle_from_graphene_rect (const graphene_rect_t *rect,
         graphene_rect_round_extents (&clamped, &clamped);
 
         *dest = (MtkRectangle) {
-          .x = clamped.origin.x,
-          .y = clamped.origin.y,
-          .width = clamped.size.width,
-          .height = clamped.size.height,
+          .x = (int) clamped.origin.x,
+          .y = (int) clamped.origin.y,
+          .width = (int) clamped.size.width,
+          .height = (int) clamped.size.height,
         };
       }
       break;
     case MTK_ROUNDING_STRATEGY_ROUND:
       {
         *dest = (MtkRectangle) {
-          .x = roundf (rect->origin.x),
-          .y = roundf (rect->origin.y),
-          .width = roundf (rect->size.width),
-          .height = roundf (rect->size.height),
+          .x = (int) roundf (rect->origin.x),
+          .y = (int) roundf (rect->origin.y),
+          .width = (int) roundf (rect->size.width),
+          .height = (int) roundf (rect->size.height),
         };
       }
     }
@@ -384,7 +418,7 @@ mtk_rectangle_scale_double (const MtkRectangle  *rect,
   graphene_rect_t tmp = GRAPHENE_RECT_INIT (rect->x, rect->y,
                                             rect->width, rect->height);
 
-  graphene_rect_scale (&tmp, scale, scale, &tmp);
+  graphene_rect_scale (&tmp, (float) scale, (float) scale, &tmp);
   mtk_rectangle_from_graphene_rect (&tmp, rounding_strategy, dest);
 }
 
@@ -410,3 +444,87 @@ mtk_rectangle_is_adjacent_to (const MtkRectangle *rect,
   else
     return FALSE;
 }
+
+/**
+ * mtk_rectangle_transform:
+ * @rect: the #MtkRectangle to be transformed
+ * @transform: the #MtkMonitorTransform
+ * @width: the width of the target space
+ * @height: the height of the target space
+ * @dest: the transformed #MtkRectangle
+ *
+ * This function transforms the values in @rect in order to compensate for
+ * @transform applied to a #MetaMonitor, making them match the viewport. Note
+ * that compensating implies that for a clockwise rotation of the #MetaMonitor
+ * an anti-clockwise rotation has to be applied to @rect.
+ */
+void
+mtk_rectangle_transform (const MtkRectangle  *rect,
+                         MtkMonitorTransform  transform,
+                         int                  width,
+                         int                  height,
+                         MtkRectangle        *dest)
+{
+  switch (transform)
+    {
+    case MTK_MONITOR_TRANSFORM_NORMAL:
+      *dest = *rect;
+      break;
+    case MTK_MONITOR_TRANSFORM_90:
+      *dest = (MtkRectangle) {
+        .x = rect->y,
+        .y = height - (rect->x + rect->width),
+        .width = rect->height,
+        .height = rect->width,
+      };
+      break;
+    case MTK_MONITOR_TRANSFORM_180:
+      *dest = (MtkRectangle) {
+        .x = width - (rect->x + rect->width),
+        .y = height - (rect->y + rect->height),
+        .width = rect->width,
+        .height = rect->height,
+      };
+      break;
+    case MTK_MONITOR_TRANSFORM_270:
+      *dest = (MtkRectangle) {
+        .x = width - (rect->y + rect->height),
+        .y = rect->x,
+        .width = rect->height,
+        .height = rect->width,
+      };
+      break;
+    case MTK_MONITOR_TRANSFORM_FLIPPED:
+      *dest = (MtkRectangle) {
+        .x = width - (rect->x + rect->width),
+        .y = rect->y,
+        .width = rect->width,
+        .height = rect->height,
+      };
+      break;
+    case MTK_MONITOR_TRANSFORM_FLIPPED_90:
+      *dest = (MtkRectangle) {
+        .x = rect->y,
+        .y = rect->x,
+        .width = rect->height,
+        .height = rect->width,
+      };
+      break;
+    case MTK_MONITOR_TRANSFORM_FLIPPED_180:
+      *dest = (MtkRectangle) {
+        .x = rect->x,
+        .y = height - (rect->y + rect->height),
+        .width = rect->width,
+        .height = rect->height,
+      };
+      break;
+    case MTK_MONITOR_TRANSFORM_FLIPPED_270:
+      *dest = (MtkRectangle) {
+        .x = width - (rect->y + rect->height),
+        .y = height - (rect->x + rect->width),
+        .width = rect->height,
+        .height = rect->width,
+      };
+      break;
+    }
+}
