@@ -30,6 +30,8 @@ typedef struct _MetaColorManager MetaColorManager;
 typedef struct _MetaColorProfile MetaColorProfile;
 typedef struct _MetaColorStore MetaColorStore;
 
+typedef enum _MetaColorMode MetaColorMode;
+
 typedef struct _MetaMonitorManager MetaMonitorManager;
 
 typedef struct _MetaMonitorConfigManager MetaMonitorConfigManager;
@@ -95,3 +97,6 @@ typedef struct _MetaInputCaptureSession MetaInputCaptureSession;
 
 typedef struct _MetaEis MetaEis;
 typedef struct _MetaEisClient MetaEisClient;
+
+typedef struct _MetaLauncher MetaLauncher;
+typedef struct _MetaUdev MetaUdev;
