@@ -22,8 +22,10 @@
 #include <glib-object.h>
 
 #include "backends/meta-backend-types.h"
+#include "core/util-private.h"
 
 #define META_TYPE_DBUS_SESSION_MANAGER (meta_dbus_session_manager_get_type ())
+META_EXPORT_TEST
 G_DECLARE_DERIVABLE_TYPE (MetaDbusSessionManager,
                           meta_dbus_session_manager,
                           META, DBUS_SESSION_MANAGER,
@@ -39,6 +41,7 @@ MetaDbusSession * meta_dbus_session_manager_create_session (MetaDbusSessionManag
                                                             GError                 **error,
                                                             ...);
 
+META_EXPORT_TEST
 MetaDbusSession * meta_dbus_session_manager_get_session (MetaDbusSessionManager *session_manager,
                                                          const char             *session_id);
 
@@ -51,3 +54,6 @@ MetaBackend * meta_dbus_session_manager_get_backend (MetaDbusSessionManager *ses
 GDBusConnection * meta_dbus_session_manager_get_connection (MetaDbusSessionManager *session_manager);
 
 GDBusInterfaceSkeleton * meta_dbus_session_manager_get_interface_skeleton (MetaDbusSessionManager *session_manager);
+
+META_EXPORT_TEST
+size_t meta_dbus_session_manager_get_num_sessions (MetaDbusSessionManager *session_manager);
