@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -24,6 +22,7 @@
 
 #include "backends/meta-screen-cast-monitor-stream.h"
 
+#include "backends/meta-eis.h"
 #include "backends/meta-logical-monitor.h"
 #include "backends/meta-screen-cast-monitor-stream-src.h"
 
@@ -44,9 +43,13 @@ struct _MetaScreenCastMonitorStream
   MetaLogicalMonitor *logical_monitor;
 };
 
-G_DEFINE_TYPE (MetaScreenCastMonitorStream,
-               meta_screen_cast_monitor_stream,
-               META_TYPE_SCREEN_CAST_STREAM)
+static void meta_eis_viewport_iface_init (MetaEisViewportInterface *eis_viewport_iface);
+
+G_DEFINE_TYPE_WITH_CODE (MetaScreenCastMonitorStream,
+                         meta_screen_cast_monitor_stream,
+                         META_TYPE_SCREEN_CAST_STREAM,
+                         G_IMPLEMENT_INTERFACE (META_TYPE_EIS_VIEWPORT,
+                                                meta_eis_viewport_iface_init))
 
 static gboolean
 update_monitor (MetaScreenCastMonitorStream *monitor_stream,
@@ -58,8 +61,8 @@ update_monitor (MetaScreenCastMonitorStream *monitor_stream,
   if (!new_logical_monitor)
     return FALSE;
 
-  if (!meta_rectangle_equal (&new_logical_monitor->rect,
-                             &monitor_stream->logical_monitor->rect))
+  if (!mtk_rectangle_equal (&new_logical_monitor->rect,
+                            &monitor_stream->logical_monitor->rect))
     return FALSE;
 
   g_set_object (&monitor_stream->monitor, new_monitor);
@@ -131,6 +134,7 @@ meta_screen_cast_monitor_stream_new (MetaScreenCastSession     *session,
                                    "connection", connection,
                                    "cursor-mode", cursor_mode,
                                    "flags", flags,
+                                   "is-configured", TRUE,
                                    "monitor", monitor,
                                    NULL);
   if (!monitor_stream)
@@ -167,7 +171,7 @@ meta_screen_cast_monitor_stream_set_parameters (MetaScreenCastStream *stream,
 {
   MetaScreenCastMonitorStream *monitor_stream =
     META_SCREEN_CAST_MONITOR_STREAM (stream);
-  MetaRectangle logical_monitor_layout;
+  MtkRectangle logical_monitor_layout;
   const char *output_name;
 
   logical_monitor_layout =
@@ -199,7 +203,7 @@ meta_screen_cast_monitor_stream_transform_position (MetaScreenCastStream *stream
   MetaScreenCastMonitorStream *monitor_stream =
     META_SCREEN_CAST_MONITOR_STREAM (stream);
   MetaBackend *backend = meta_monitor_get_backend (monitor_stream->monitor);
-  MetaRectangle logical_monitor_layout;
+  MtkRectangle logical_monitor_layout;
   double scale;
 
   logical_monitor_layout =
@@ -216,6 +220,81 @@ meta_screen_cast_monitor_stream_transform_position (MetaScreenCastStream *stream
   return TRUE;
 }
 
+static gboolean
+meta_screen_cast_monitor_stream_is_standalone (MetaEisViewport *viewport)
+{
+  return FALSE;
+}
+
+static const char *
+meta_screen_cast_monitor_stream_get_mapping_id (MetaEisViewport *viewport)
+{
+  MetaScreenCastStream *stream = META_SCREEN_CAST_STREAM (viewport);
+
+  return meta_screen_cast_stream_get_mapping_id (stream);
+}
+
+static gboolean
+meta_screen_cast_monitor_stream_get_position (MetaEisViewport *viewport,
+                                              int             *out_x,
+                                              int             *out_y)
+{
+  MetaScreenCastMonitorStream *monitor_stream =
+    META_SCREEN_CAST_MONITOR_STREAM (viewport);
+  MtkRectangle layout;
+
+  layout = meta_logical_monitor_get_layout (monitor_stream->logical_monitor);
+  *out_x = layout.x;
+  *out_y = layout.y;
+  return TRUE;
+}
+
+static void
+meta_screen_cast_monitor_stream_get_size (MetaEisViewport *viewport,
+                                          int             *out_width,
+                                          int             *out_height)
+{
+  MetaScreenCastMonitorStream *monitor_stream =
+    META_SCREEN_CAST_MONITOR_STREAM (viewport);
+  MtkRectangle layout;
+
+  layout = meta_logical_monitor_get_layout (monitor_stream->logical_monitor);
+  *out_width = layout.width;
+  *out_height = layout.height;
+}
+
+static double
+meta_screen_cast_monitor_stream_get_physical_scale (MetaEisViewport *viewport)
+{
+  MetaScreenCastMonitorStream *monitor_stream =
+    META_SCREEN_CAST_MONITOR_STREAM (viewport);
+
+  return meta_logical_monitor_get_scale (monitor_stream->logical_monitor);
+}
+
+static gboolean
+meta_screen_cast_monitor_stream_transform_coordinate (MetaEisViewport *viewport,
+                                                      double           x,
+                                                      double           y,
+                                                      double          *out_x,
+                                                      double          *out_y)
+{
+  *out_x = x;
+  *out_y = y;
+  return TRUE;
+}
+
+static void
+meta_eis_viewport_iface_init (MetaEisViewportInterface *eis_viewport_iface)
+{
+  eis_viewport_iface->is_standalone = meta_screen_cast_monitor_stream_is_standalone;
+  eis_viewport_iface->get_mapping_id = meta_screen_cast_monitor_stream_get_mapping_id;
+  eis_viewport_iface->get_position = meta_screen_cast_monitor_stream_get_position;
+  eis_viewport_iface->get_size = meta_screen_cast_monitor_stream_get_size;
+  eis_viewport_iface->get_physical_scale = meta_screen_cast_monitor_stream_get_physical_scale;
+  eis_viewport_iface->transform_coordinate = meta_screen_cast_monitor_stream_transform_coordinate;
+}
+
 static void
 meta_screen_cast_monitor_stream_set_property (GObject      *object,
                                               guint         prop_id,
@@ -291,9 +370,7 @@ meta_screen_cast_monitor_stream_class_init (MetaScreenCastMonitorStreamClass *kl
 
   g_object_class_install_property (object_class,
                                    PROP_MONITOR,
-                                   g_param_spec_object ("monitor",
-                                                        "monitor",
-                                                        "MetaMonitor",
+                                   g_param_spec_object ("monitor", NULL, NULL,
                                                         META_TYPE_MONITOR,
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY |
