@@ -118,8 +118,9 @@ MetaScreenCastStream * meta_screen_cast_stream_src_get_stream (MetaScreenCastStr
 
 gboolean meta_screen_cast_stream_src_draw_cursor_into (MetaScreenCastStreamSrc  *src,
                                                        CoglTexture              *cursor_texture,
-                                                       float                     scale,
-                                                       MtkMonitorTransform       transform,
+                                                       int                       width,
+                                                       int                       height,
+                                                       const graphene_matrix_t  *matrix,
                                                        uint8_t                  *data,
                                                        GError                  **error);
 
@@ -141,8 +142,7 @@ void meta_screen_cast_stream_src_set_cursor_sprite_metadata (MetaScreenCastStrea
                                                              MetaCursorSprite        *cursor_sprite,
                                                              int                      x,
                                                              int                      y,
-                                                             float                    scale,
-                                                             MtkMonitorTransform      transform);
+                                                             float                    view_scale);
 
 gboolean meta_screen_cast_stream_src_uses_dma_bufs (MetaScreenCastStreamSrc *src);
 
