@@ -51,7 +51,6 @@
 #include "backends/native/meta-gpu-kms.h"
 #include "backends/native/meta-kms-device.h"
 #include "backends/native/meta-kms.h"
-#include "backends/native/meta-launcher.h"
 #include "backends/native/meta-output-kms.h"
 #include "backends/native/meta-renderer-native.h"
 #include "backends/native/meta-virtual-monitor-native.h"
@@ -100,27 +99,6 @@ meta_monitor_manager_native_read_edid (MetaMonitorManager *manager,
   return meta_output_native_read_edid (META_OUTPUT_NATIVE (output));
 }
 
-static void
-meta_monitor_manager_native_read_current_state (MetaMonitorManager *manager)
-{
-  MetaMonitorManagerClass *parent_class =
-    META_MONITOR_MANAGER_CLASS (meta_monitor_manager_native_parent_class);
-  MetaPowerSave power_save_mode;
-
-  power_save_mode = meta_monitor_manager_get_power_save_mode (manager);
-  if (power_save_mode != META_POWER_SAVE_ON)
-    {
-      MetaPowerSaveChangeReason reason;
-
-      reason = META_POWER_SAVE_CHANGE_REASON_HOTPLUG;
-      meta_monitor_manager_power_save_mode_changed (manager,
-                                                    META_POWER_SAVE_ON,
-                                                    reason);
-    }
-
-  parent_class->read_current_state (manager);
-}
-
 static void
 meta_monitor_manager_native_set_power_save_mode (MetaMonitorManager *manager,
                                                  MetaPowerSave       mode)
@@ -156,6 +134,7 @@ meta_monitor_manager_native_ensure_initial_config (MetaMonitorManager *manager)
   config = meta_monitor_manager_ensure_configured (manager);
 
   meta_monitor_manager_update_logical_state (manager, config);
+  meta_monitor_manager_update_for_lease_state (manager, config);
 }
 
 static void
@@ -557,7 +536,7 @@ allocate_virtual_monitor_id (MetaMonitorManagerNative *manager_native)
     }
 }
 
-static gboolean
+static void
 rebuild_virtual_idle_cb (gpointer user_data)
 {
   MetaMonitorManager *manager = user_data;
@@ -569,8 +548,6 @@ rebuild_virtual_idle_cb (gpointer user_data)
   priv->rebuild_virtual_idle_id = 0;
 
   meta_monitor_manager_reconfigure (manager);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -586,7 +563,7 @@ on_virtual_monitor_mode_changed (MetaVirtualMonitor *virtual_monitor,
   if (priv->rebuild_virtual_idle_id)
     return;
 
-  priv->rebuild_virtual_idle_id = g_idle_add (rebuild_virtual_idle_cb, manager);
+  priv->rebuild_virtual_idle_id = g_idle_add_once (rebuild_virtual_idle_cb, manager);
 }
 
 static MetaVirtualMonitor *
@@ -714,8 +691,6 @@ meta_monitor_manager_native_class_init (MetaMonitorManagerNativeClass *klass)
 
   manager_class->read_edid =
     meta_monitor_manager_native_read_edid;
-  manager_class->read_current_state =
-    meta_monitor_manager_native_read_current_state;
   manager_class->ensure_initial_config =
     meta_monitor_manager_native_ensure_initial_config;
   manager_class->apply_monitors_config =
