@@ -374,7 +374,7 @@ get_closest_border (GArray    *borders,
   MetaVector2 delta;
   float distance_2;
   MetaBorder *closest_border = NULL;
-  float closest_distance_2 = DBL_MAX;
+  float closest_distance_2 = FLT_MAX;
   unsigned int i;
 
   for (i = 0; i < borders->len; i++)
@@ -408,7 +408,7 @@ clamp_to_border (MetaBorder *border,
   if (meta_border_is_horizontal (border))
     {
       if (*motion_dir & META_BORDER_MOTION_DIRECTION_POSITIVE_Y)
-        motion->b.y = border->line.a.y - min_edge_distance;
+        motion->b.y = (float) (border->line.a.y - min_edge_distance);
       else
         motion->b.y = border->line.a.y;
       *motion_dir &= ~(META_BORDER_MOTION_DIRECTION_POSITIVE_Y |
@@ -417,7 +417,7 @@ clamp_to_border (MetaBorder *border,
   else
     {
       if (*motion_dir & META_BORDER_MOTION_DIRECTION_POSITIVE_X)
-        motion->b.x = border->line.a.x - min_edge_distance;
+        motion->b.x = (float) (border->line.a.x - min_edge_distance);
       else
         motion->b.x = border->line.a.x;
       *motion_dir &= ~(META_BORDER_MOTION_DIRECTION_POSITIVE_X |
@@ -570,24 +570,24 @@ closest_point_behind_border (MetaBorder *border,
     case META_BORDER_MOTION_DIRECTION_POSITIVE_X:
     case META_BORDER_MOTION_DIRECTION_NEGATIVE_X:
       if (border->blocking_directions == META_BORDER_MOTION_DIRECTION_POSITIVE_X)
-        *sx = border->line.a.x - wl_fixed_to_double (1);
+        *sx = border->line.a.x - (float) wl_fixed_to_double (1);
       else
-        *sx = border->line.a.x + wl_fixed_to_double (1);
+        *sx = border->line.a.x + (float) wl_fixed_to_double (1);
       if (*sy < border->line.a.y)
-        *sy = border->line.a.y + wl_fixed_to_double (1);
+        *sy = border->line.a.y + (float) wl_fixed_to_double (1);
       else if (*sy > border->line.b.y)
-        *sy = border->line.b.y - wl_fixed_to_double (1);
+        *sy = border->line.b.y - (float) wl_fixed_to_double (1);
       break;
     case META_BORDER_MOTION_DIRECTION_POSITIVE_Y:
     case META_BORDER_MOTION_DIRECTION_NEGATIVE_Y:
       if (border->blocking_directions == META_BORDER_MOTION_DIRECTION_POSITIVE_Y)
-        *sy = border->line.a.y - wl_fixed_to_double (1);
+        *sy = border->line.a.y - (float) wl_fixed_to_double (1);
       else
-        *sy = border->line.a.y + wl_fixed_to_double (1);
+        *sy = border->line.a.y + (float) wl_fixed_to_double (1);
       if (*sx < border->line.a.x)
-        *sx = border->line.a.x + wl_fixed_to_double (1);
+        *sx = border->line.a.x + (float) wl_fixed_to_double (1);
       else if (*sx > (border->line.b.x))
-        *sx = border->line.b.x - wl_fixed_to_double (1);
+        *sx = border->line.b.x - (float) wl_fixed_to_double (1);
       break;
     }
 }
@@ -621,8 +621,8 @@ meta_pointer_constraint_impl_native_ensure_constrained (MetaPointerConstraintImp
           y != constraint_impl_native->origin.y)
         {
           clutter_seat_warp_pointer (seat,
-                                     constraint_impl_native->origin.x,
-                                     constraint_impl_native->origin.y);
+                                     (int) constraint_impl_native->origin.x,
+                                     (int) constraint_impl_native->origin.y);
         }
     }
   else if (!mtk_region_contains_point (region, (int) rel_x, (int) rel_y))
@@ -652,8 +652,8 @@ meta_pointer_constraint_impl_native_ensure_constrained (MetaPointerConstraintImp
       closest_point_behind_border (closest_border, &rel_x, &rel_y);
 
       clutter_seat_warp_pointer (seat,
-                                 rel_x + constraint_impl_native->origin.x,
-                                 rel_y + constraint_impl_native->origin.y);
+                                 (int) (rel_x + constraint_impl_native->origin.x),
+                                 (int) (rel_y + constraint_impl_native->origin.y));
     }
 }
 
