@@ -103,7 +103,8 @@ void  meta_seat_native_reclaim_devices (MetaSeatNative *seat);
 void meta_seat_native_set_keyboard_map (MetaSeatNative *seat,
                                         const char     *layouts,
                                         const char     *variants,
-                                        const char     *options);
+                                        const char     *options,
+                                        const char     *model);
 
 struct xkb_keymap * meta_seat_native_get_keyboard_map (MetaSeatNative *seat);
 
@@ -136,3 +137,7 @@ void meta_seat_native_run_impl_task (MetaSeatNative *seat,
                                      GSourceFunc     dispatch_func,
                                      gpointer        user_data,
                                      GDestroyNotify  destroy_notify);
+
+void meta_seat_native_set_a11y_modifiers (MetaSeatNative *seat,
+                                          const uint32_t *modifiers,
+                                          int             n_modifiers);
