@@ -14,13 +14,10 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_GPU_XRANDR_H
-#define META_GPU_XRANDR_H
+#pragma once
 
 #include <glib-object.h>
 #include <X11/extensions/Xrandr.h>
@@ -38,5 +35,3 @@ void meta_gpu_xrandr_get_max_screen_size (MetaGpuXrandr *gpu_xrandr,
                                           int           *max_height);
 
 MetaGpuXrandr * meta_gpu_xrandr_new (MetaBackendX11 *backend_x11);
-
-#endif /* META_GPU_XRANDR_H */
