@@ -422,6 +422,68 @@ get_pad_features (XIDeviceInfo *info,
   *n_strips = strips;
 }
 
+static gboolean
+guess_serial_from_libinput_prop (MetaSeatX11        *seat_x11,
+                                 ClutterInputDevice *device,
+                                 unsigned int       *serial_out)
+{
+  Display *xdisplay = xdisplay_from_seat (seat_x11);
+  int device_id = meta_input_device_x11_get_device_id (device);
+  gulong nitems, bytes_after;
+  uint32_t *data = NULL;
+  int rc, format;
+  Atom type;
+  Atom prop;
+
+  prop = XInternAtom (xdisplay, "libinput Tablet Tool Serial", True);
+  if (prop != None)
+    {
+      mtk_x11_error_trap_push (xdisplay);
+      rc = XIGetProperty (xdisplay,
+                          device_id,
+                          prop,
+                          0, 1, False, XA_CARDINAL, &type, &format, &nitems, &bytes_after,
+                          (guchar **) &data);
+      mtk_x11_error_trap_pop (xdisplay);
+
+      if (rc != Success || type != XA_CARDINAL || format != 32 || nitems != 1)
+        {
+          XFree (data);
+          return FALSE;
+        }
+
+      *serial_out = *data;
+      XFree (data);
+      return TRUE;
+    }
+
+  /* xf86-input-libinput < 1.5.0 doesn't have the serial prop, let's check
+   * for a generic property that tells us if it's libinput handling this device.
+   * If that's the case we give the tool a "random" fixed serial to pass through
+   * the code that assumes 0 == no tool.
+   */
+  prop = XInternAtom (xdisplay, "libinput Send Events Mode Enabled", True);
+  if (prop == None)
+    return FALSE;
+
+  mtk_x11_error_trap_push (xdisplay);
+  rc = XIGetProperty (xdisplay,
+                      device_id,
+                      prop,
+                      0, 1, False, XA_INTEGER, &type, &format, &nitems, &bytes_after,
+                      (guchar **) &data);
+  mtk_x11_error_trap_pop (xdisplay);
+  XFree (data);
+
+  if (rc != Success)
+      return FALSE;
+
+  *serial_out = 0xffffffaa;
+
+  return TRUE;
+}
+
+
 /* The Wacom driver exports the tool type as property. Use that over
    guessing based on the device name */
 static gboolean
@@ -807,6 +869,47 @@ remove_device (MetaSeatX11        *seat_x11,
     }
 }
 
+static void
+update_tool (MetaSeatX11        *seat_x11,
+             ClutterInputDevice *device,
+             int                 serial_id)
+{
+  ClutterInputDeviceTool *tool = NULL;
+  ClutterInputDeviceToolType type;
+  MetaInputSettings *input_settings;
+  int lookup_serial;
+
+  if (serial_id != 0)
+    {
+      /* stylus and eraser share the same serial, so bitflip the eraser's
+       * serial to make them distinct in the hashtable */
+      if (clutter_input_device_get_device_type (device) == CLUTTER_ERASER_DEVICE)
+        {
+          lookup_serial = ~serial_id;
+          type = CLUTTER_INPUT_DEVICE_TOOL_ERASER;
+        }
+      else
+        {
+          lookup_serial = serial_id;
+          type = CLUTTER_INPUT_DEVICE_TOOL_PEN;
+        }
+
+      tool = g_hash_table_lookup (seat_x11->tools_by_serial,
+                                  GUINT_TO_POINTER (lookup_serial));
+      if (!tool)
+        {
+          tool = meta_input_device_tool_x11_new (serial_id, type);
+          g_hash_table_insert (seat_x11->tools_by_serial,
+                               GUINT_TO_POINTER (lookup_serial),
+                               tool);
+        }
+    }
+
+  meta_input_device_x11_update_tool (device, tool);
+  input_settings = meta_backend_get_input_settings (seat_x11->backend);
+  meta_input_settings_notify_tool_change (input_settings, device, tool);
+}
+
 static gboolean
 meta_seat_x11_handle_event_post (ClutterSeat        *seat,
                                  const ClutterEvent *event)
@@ -815,7 +918,9 @@ meta_seat_x11_handle_event_post (ClutterSeat        *seat,
   ClutterInputDevice *device;
   MetaInputSettings *input_settings;
   ClutterEventType event_type;
-  gboolean is_touch;
+  gboolean is_touch, is_tablet_tool;
+  unsigned int serial;
+  ClutterInputDeviceType type;
 
   event_type = clutter_event_type (event);
 
@@ -824,8 +929,9 @@ meta_seat_x11_handle_event_post (ClutterSeat        *seat,
     return TRUE;
 
   device = clutter_event_get_device (event);
-  is_touch =
-    clutter_input_device_get_device_type (device) == CLUTTER_TOUCHSCREEN_DEVICE;
+  type = clutter_input_device_get_device_type (device);
+  is_touch = type == CLUTTER_TOUCHSCREEN_DEVICE;
+  is_tablet_tool = type == CLUTTER_PEN_DEVICE || type == CLUTTER_ERASER_DEVICE;
   input_settings = meta_backend_get_input_settings (seat_x11->backend);
 
   switch (event_type)
@@ -833,6 +939,8 @@ meta_seat_x11_handle_event_post (ClutterSeat        *seat,
       case CLUTTER_DEVICE_ADDED:
         meta_input_settings_add_device (input_settings, device);
         seat_x11->has_touchscreens |= is_touch;
+        if (is_tablet_tool && guess_serial_from_libinput_prop (seat_x11, device, &serial))
+          update_tool(seat_x11, device, serial);
         break;
       case CLUTTER_DEVICE_REMOVED:
         if (is_touch)
@@ -971,31 +1079,10 @@ translate_property_event (MetaSeatX11 *seat_x11,
 
   if (xev->property == serial_ids_prop)
     {
-      ClutterInputDeviceTool *tool = NULL;
-      ClutterInputDeviceToolType type;
-      MetaInputSettings *input_settings;
       int serial_id;
 
       serial_id = device_get_tool_serial (seat_x11, device);
-
-      if (serial_id != 0)
-        {
-          tool = g_hash_table_lookup (seat_x11->tools_by_serial,
-                                      GUINT_TO_POINTER (serial_id));
-          if (!tool)
-            {
-              type = clutter_input_device_get_device_type (device) == CLUTTER_ERASER_DEVICE ?
-                CLUTTER_INPUT_DEVICE_TOOL_ERASER : CLUTTER_INPUT_DEVICE_TOOL_PEN;
-              tool = meta_input_device_tool_x11_new (serial_id, type);
-              g_hash_table_insert (seat_x11->tools_by_serial,
-                                   GUINT_TO_POINTER (serial_id),
-                                   tool);
-            }
-        }
-
-      meta_input_device_x11_update_tool (device, tool);
-      input_settings = meta_backend_get_input_settings (seat_x11->backend);
-      meta_input_settings_notify_tool_change (input_settings, device, tool);
+      update_tool(seat_x11, device, serial_id);
     }
 }
 
@@ -1013,7 +1100,7 @@ emulate_motion (MetaSeatX11 *seat_x11,
                                     CLUTTER_CURRENT_TIME,
                                     pointer,
                                     NULL, 0,
-                                    GRAPHENE_POINT_INIT (x, y),
+                                    GRAPHENE_POINT_INIT ((float) x, (float) y),
                                     GRAPHENE_POINT_INIT (0, 0),
                                     GRAPHENE_POINT_INIT (0, 0),
                                     GRAPHENE_POINT_INIT (0, 0),
@@ -1393,8 +1480,8 @@ translate_coords (MetaStageX11 *stage_x11,
 
   clutter_actor_get_size (stage, &stage_width, &stage_height);
 
-  *x_out = CLAMP (event_x, 0, stage_width);
-  *y_out = CLAMP (event_y, 0, stage_height);
+  *x_out = (float) CLAMP (event_x, 0, stage_width);
+  *y_out = (float) CLAMP (event_y, 0, stage_height);
 }
 
 static void
@@ -1771,16 +1858,16 @@ meta_seat_x11_query_state (ClutterSeat          *seat,
 
       if (coords)
         {
-          coords->x = touch_info->x;
-          coords->y = touch_info->y;
+          coords->x = (float) touch_info->x;
+          coords->y = (float) touch_info->y;
         }
     }
   else
     {
       if (coords)
         {
-          coords->x = win_x;
-          coords->y = win_y;
+          coords->x = (float) win_x;
+          coords->y = (float) win_y;
         }
     }
 
@@ -1847,6 +1934,8 @@ meta_seat_x11_grab (ClutterSeat *seat,
 
   seat_x11->grab_state = state;
 
+  meta_backend_x11_sync_pointer (META_BACKEND_X11 (backend));
+
   return state;
 }
 
@@ -1872,6 +1961,8 @@ meta_seat_x11_ungrab (ClutterSeat *seat,
     }
 
   seat_x11->grab_state = CLUTTER_GRAB_STATE_NONE;
+
+  meta_backend_x11_sync_pointer (META_BACKEND_X11 (backend));
 }
 
 static void
@@ -1945,9 +2036,12 @@ meta_seat_x11_new (MetaBackend *backend,
                    int          logical_keyboard)
 {
   MetaSeatX11 *seat_x11;
+  ClutterContext *clutter_context =
+    meta_backend_get_clutter_context (backend);
 
   seat_x11 = g_object_new (META_TYPE_SEAT_X11,
                            "backend", backend,
+                           "context", clutter_context,
                            "opcode", opcode,
                            "pointer-id", logical_pointer,
                            "keyboard-id", logical_keyboard,
@@ -2111,6 +2205,7 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
         MetaKeymapX11 *keymap_x11 = seat->keymap;
         char buffer[7] = { 0, };
         uint32_t keyval, evcode, keycode;
+        ClutterModifierSet raw_modifiers;
         ClutterModifierType state;
         int len;
         gunichar unicode_value;
@@ -2119,6 +2214,12 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
         if (!source_device)
           return NULL;
 
+        raw_modifiers = (ClutterModifierSet) {
+          .pressed = xev->mods.base,
+          .latched = xev->mods.latched,
+          .locked = xev->mods.locked,
+        };
+
         state = translate_state (&xev->buttons, &xev->mods, &xev->group);
 
         keycode = xev->detail;
@@ -2156,6 +2257,7 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
                                        CLUTTER_EVENT_NONE,
                                        ms2us (xev->time),
                                        source_device,
+                                       raw_modifiers,
                                        state,
                                        keyval,
                                        evcode,
@@ -2198,7 +2300,7 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
 
         if (clutter_input_device_get_device_type (source_device) == CLUTTER_PAD_DEVICE)
           {
-            uint32_t button, group = 0, mode = 0;
+            uint32_t pad_button, group = 0, mode = 0;
 
             /* We got these events because of the passive button grab */
             XIAllowEvents (xdisplay, xev->sourceid, XIAsyncDevice, xev->time);
@@ -2217,11 +2319,11 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
               xev->detail -= 4;
 
             /* Pad buttons are 0-indexed */
-            button = xev->detail - 1;
+            pad_button = xev->detail - 1;
 
 #ifdef HAVE_LIBWACOM
             meta_input_device_x11_update_pad_state (device,
-                                                    button,
+                                                    pad_button,
                                                     (xi_event->evtype == XI_ButtonPress),
                                                     &group,
                                                     &mode);
@@ -2233,7 +2335,7 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
                                                   CLUTTER_EVENT_NONE,
                                                   us2ms (xev->time),
                                                   source_device,
-                                                  button,
+                                                  pad_button,
                                                   group,
                                                   mode);
 
@@ -2246,7 +2348,7 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
                      meta_input_device_x11_get_device_id (device),
                      clutter_input_device_get_device_name (device),
                      xev->time,
-                     button);
+                     pad_button);
             break;
           }
 
@@ -2279,6 +2381,7 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
                                                        tool,
                                                        state,
                                                        GRAPHENE_POINT_INIT (x, y),
+                                                       CLUTTER_SCROLL_SOURCE_UNKNOWN,
                                                        scroll_direction);
 
             g_debug ("scroll: win:0x%x, device:%d '%s', time:%d "
@@ -2398,7 +2501,8 @@ meta_seat_x11_translate_event (MetaSeatX11  *seat,
                                                      tool,
                                                      state,
                                                      GRAPHENE_POINT_INIT (x, y),
-                                                     GRAPHENE_POINT_INIT (delta_x, delta_y),
+                                                     GRAPHENE_POINT_INIT ((float) delta_x,
+                                                                          (float) delta_y),
                                                      CLUTTER_SCROLL_SOURCE_UNKNOWN,
                                                      CLUTTER_SCROLL_FINISHED_NONE);
 
