@@ -25,6 +25,9 @@
 
 #include <math.h>
 
+#define FIXED_SHIFT 8
+#define FIXED_ONE (1 << FIXED_SHIFT)
+
 /* This file uses pixel-aligned region computation to determine what
  * can be clipped out. This only really works if everything is aligned
  * to the pixel grid - not scaled or rotated and at integer offsets.
@@ -46,12 +49,12 @@
  */
 
 /* The definition of "close enough" to integral pixel values is
- * equality when we convert to 24.8 fixed-point.
+ * equality when we convert to fixed-point.
  */
 static inline int
 round_to_fixed (float x)
 {
-  return roundf (x * 256);
+  return (int) roundf (x * FIXED_ONE);
 }
 
 /* Help macros to scale from OpenGL <-1,1> coordinates system to
@@ -69,8 +72,7 @@ gboolean
 meta_actor_vertices_are_untransformed (graphene_point3d_t *verts,
                                        float               widthf,
                                        float               heightf,
-                                       int                *x_origin,
-                                       int                *y_origin)
+                                       MetaTransforms     *out_transforms)
 {
   int width, height;
   int v0x, v0y, v1x, v1y, v2x, v2y, v3x, v3y;
@@ -85,13 +87,21 @@ meta_actor_vertices_are_untransformed (graphene_point3d_t *verts,
   v3x = round_to_fixed (verts[3].x); v3y = round_to_fixed (verts[3].y);
 
   /* Using shifting for converting fixed => int, gets things right for
-   * negative values. / 256. wouldn't do the same
+   * negative values. / FIXED_ONE wouldn't do the same
    */
-  x = v0x >> 8;
-  y = v0y >> 8;
+  x = v0x >> FIXED_SHIFT;
+  y = v0y >> FIXED_SHIFT;
+
+  if (out_transforms)
+    {
+      out_transforms->x_origin = x;
+      out_transforms->y_origin = y;
+      out_transforms->x_scale = (v1x - v0x) / (float) width;
+      out_transforms->y_scale = (v2y - v0y) / (float) height;
+    }
 
   /* At integral coordinates? */
-  if (x * 256 != v0x || y * 256 != v0y)
+  if (x * FIXED_ONE != v0x || y * FIXED_ONE != v0y)
     return FALSE;
 
   /* Not scaled? */
@@ -103,11 +113,6 @@ meta_actor_vertices_are_untransformed (graphene_point3d_t *verts,
       v3x != v1x || v3y != v2y)
     return FALSE;
 
-  if (x_origin)
-    *x_origin = x;
-  if (y_origin)
-    *y_origin = y;
-
   return TRUE;
 }
 
@@ -137,8 +142,7 @@ meta_actor_painting_untransformed (CoglFramebuffer *fb,
                                    int              paint_height,
                                    int              sample_width,
                                    int              sample_height,
-                                   int             *x_origin,
-                                   int             *y_origin)
+                                   MetaTransforms  *out_transforms)
 {
   graphene_matrix_t modelview, projection, modelview_projection;
   graphene_point3d_t vertices[4];
@@ -183,6 +187,6 @@ meta_actor_painting_untransformed (CoglFramebuffer *fb,
 
   return meta_actor_vertices_are_untransformed (vertices,
                                                 sample_width, sample_height,
-                                                x_origin, y_origin);
+                                                out_transforms);
 }
 
