@@ -20,21 +20,21 @@ actor_meta_clear (void)
   clutter_actor_add_constraint (actor, clutter_bind_constraint_new (stage, CLUTTER_BIND_ALL, 0));
   clutter_actor_add_effect (actor, clutter_blur_effect_new ());
 
-  g_assert (clutter_actor_has_actions (actor));
-  g_assert (clutter_actor_has_constraints (actor));
-  g_assert (clutter_actor_has_effects (actor));
+  g_assert_true (clutter_actor_has_actions (actor));
+  g_assert_true (clutter_actor_has_constraints (actor));
+  g_assert_true (clutter_actor_has_effects (actor));
 
   clutter_actor_clear_actions (actor);
-  g_assert (!clutter_actor_has_actions (actor));
+  g_assert_false (clutter_actor_has_actions (actor));
 
   clutter_actor_clear_constraints (actor);
-  g_assert (!clutter_actor_has_constraints (actor));
+  g_assert_false (clutter_actor_has_constraints (actor));
 
   clutter_actor_clear_effects (actor);
-  g_assert (!clutter_actor_has_effects (actor));
+  g_assert_false (clutter_actor_has_effects (actor));
 
   clutter_actor_destroy (actor);
-  g_assert (actor == NULL);
+  g_assert_null (actor);
 }
 
 CLUTTER_TEST_SUITE (
