@@ -52,7 +52,7 @@
 #include "backends/meta-gpu.h"
 #include "backends/meta-stage-private.h"
 #include "clutter/clutter/clutter-stage-view-private.h"
-#include "meta/compositor-mutter.h"
+#include "meta/compositor.h"
 #include "tests/meta-ref-test-utils.h"
 
 static void
@@ -146,7 +146,8 @@ on_after_paint (MetaStage        *stage,
 }
 
 static cairo_surface_t *
-capture_view (ClutterStageView *stage_view)
+capture_view (ClutterStageView *stage_view,
+              gboolean          queue_damage)
 {
   MetaRendererView *view = META_RENDERER_VIEW (stage_view);
   MetaCrtc *crtc = meta_renderer_view_get_crtc (view);
@@ -154,16 +155,19 @@ capture_view (ClutterStageView *stage_view)
   MetaStage *stage = META_STAGE (meta_backend_get_stage (backend));
   MetaContext *context = meta_backend_get_context (backend);
   MetaDisplay *display = meta_context_get_display (context);
+  MetaCompositor *compositor = meta_display_get_compositor (display);
   CaptureViewData data = { 0 };
 
-  meta_disable_unredirect_for_display (display);
+  meta_compositor_disable_unredirect (compositor);
+  meta_backend_inhibit_hw_cursor (backend);
 
   data.loop = g_main_loop_new (NULL, FALSE);
   data.watch = meta_stage_watch_view (stage, stage_view,
                                       META_STAGE_WATCH_AFTER_PAINT,
                                       on_after_paint,
                                       &data);
-  clutter_stage_view_add_redraw_clip (stage_view, NULL);
+  if (queue_damage)
+    clutter_stage_view_add_redraw_clip (stage_view, NULL);
   clutter_stage_view_schedule_update (stage_view);
 
   g_main_loop_run (data.loop);
@@ -172,7 +176,8 @@ capture_view (ClutterStageView *stage_view)
   g_assert_null (data.watch);
   g_assert_nonnull (data.out_image);
 
-  meta_enable_unredirect_for_display (display);
+  meta_backend_uninhibit_hw_cursor (backend);
+  meta_compositor_enable_unredirect (compositor);
 
   return data.out_image;
 }
@@ -182,7 +187,15 @@ view_adaptor_capture (gpointer adaptor_data)
 {
   ClutterStageView *view = CLUTTER_STAGE_VIEW (adaptor_data);
 
-  return capture_view (view);
+  return capture_view (view, TRUE);
+}
+
+static cairo_surface_t *
+view_adaptor_capture_undamaged (gpointer adaptor_data)
+{
+  ClutterStageView *view = CLUTTER_STAGE_VIEW (adaptor_data);
+
+  return capture_view (view, FALSE);
 }
 
 static void
@@ -211,6 +224,22 @@ meta_ref_test_verify_view (ClutterStageView *view,
                         flags);
 }
 
+void
+meta_ref_test_verify_view_undamaged (ClutterStageView *view,
+                                     const char       *test_name_unescaped,
+                                     int               test_seq_no,
+                                     MetaReftestFlag   flags)
+{
+  if (flags & META_REFTEST_FLAG_UPDATE_REF)
+    assert_software_rendered (view);
+
+  meta_ref_test_verify (view_adaptor_capture_undamaged,
+                        view,
+                        test_name_unescaped,
+                        test_seq_no,
+                        flags);
+}
+
 MetaReftestFlag
 meta_ref_test_determine_ref_test_flag (void)
 {
