@@ -871,7 +871,7 @@ toplevel_sessions_restore_fullscreen (void)
 
   g_assert_nonnull (session_id);
 
-  /* Launch client again, check window persists maximized on second monitor */
+  /* Launch client again, check window persists fullscreen on second monitor */
   wayland_test_client =
     meta_wayland_test_client_new_with_args (test_context,
                                             "xdg-session-management-restore",
@@ -898,6 +898,183 @@ toplevel_sessions_restore_fullscreen (void)
   meta_wayland_test_client_finish (wayland_test_client);
 }
 
+static void
+toplevel_sessions_restore_maximized (void)
+{
+  MetaSessionManager *session_manager;
+  g_autoptr (MetaVirtualMonitor) second_virtual_monitor = NULL;
+  MetaBackend *backend = meta_context_get_backend (test_context);
+  ClutterActor *stage = meta_backend_get_stage (backend);
+  MetaWaylandTestClient *wayland_test_client;
+  MetaWindow *window;
+  MtkRectangle frame_rect, monitor_layout;
+  g_autofree char *session_id = NULL;
+  MetaWindowWayland *wl_window;
+  uint32_t state_change_serial = 0U;
+
+  monitor_layout = get_primary_logical_monitor_layout ();
+
+  second_virtual_monitor = meta_create_test_monitor (test_context,
+                                                     800, 600, 60.0);
+
+  session_manager = meta_context_get_session_manager (test_context);
+  g_signal_connect (session_manager, "session-instantiated",
+                    G_CALLBACK (on_session_instantiated), &session_id);
+
+  /* Launch client once, resize window */
+  wayland_test_client =
+    meta_wayland_test_client_new (test_context, "xdg-session-management-restore");
+
+  wait_for_sync_point (0);
+  wait_for_paint (stage);
+
+  window = find_client_window ("toplevel1");
+  g_assert_nonnull (window);
+
+  /* Move to second monitor */
+  meta_window_move_resize_frame (window, FALSE,
+                                 monitor_layout.width + 123, 123, 100, 100);
+  wait_for_paint (stage);
+
+  frame_rect = meta_window_config_get_rect (window->config);
+  g_assert_cmpint (frame_rect.x, ==, monitor_layout.width + 123);
+  g_assert_cmpint (frame_rect.y, ==, 123);
+  g_assert_cmpint (frame_rect.width, ==, 100);
+  g_assert_cmpint (frame_rect.height, ==, 100);
+
+  meta_window_maximize (window, META_MAXIMIZE_BOTH);
+
+  wl_window = META_WINDOW_WAYLAND (window);
+  meta_window_wayland_get_pending_serial (wl_window, &state_change_serial);
+  g_assert_cmpint (state_change_serial, !=, 0U);
+  while (meta_window_wayland_peek_configuration (wl_window, state_change_serial))
+    g_main_context_iteration (NULL, TRUE);
+
+  frame_rect = meta_window_config_get_rect (window->config);
+  g_assert_cmpint (frame_rect.x, ==, 640);
+  g_assert_cmpint (frame_rect.y, ==, 0);
+  g_assert_cmpint (frame_rect.width, ==, 800);
+  g_assert_cmpint (frame_rect.height, ==, 600);
+
+  meta_wayland_test_driver_emit_sync_event (test_driver, 0);
+  meta_wayland_test_client_finish (wayland_test_client);
+
+  g_assert_nonnull (session_id);
+
+  /* Launch client again, check window persists maximized on second monitor */
+  wayland_test_client =
+    meta_wayland_test_client_new_with_args (test_context,
+                                            "xdg-session-management-restore",
+                                            session_id,
+                                            NULL);
+
+  wait_for_sync_point (0);
+
+  window = find_client_window ("toplevel1");
+  g_assert_nonnull (window);
+
+  g_assert_nonnull (window->monitor);
+  frame_rect = meta_window_config_get_rect (window->config);
+  g_assert_cmpint (frame_rect.x, ==, monitor_layout.width);
+  g_assert_cmpint (frame_rect.y, ==, 0);
+  g_assert_cmpint (frame_rect.width, ==, 800);
+  g_assert_cmpint (frame_rect.height, ==, 600);
+
+  g_signal_handlers_disconnect_by_func (session_manager, on_session_instantiated,
+                                        &session_id);
+
+  meta_wayland_test_driver_emit_sync_event (test_driver, 0);
+  meta_wayland_test_client_finish (wayland_test_client);
+}
+
+static void
+toplevel_sessions_restore_tiled (void)
+{
+  MetaSessionManager *session_manager;
+  g_autoptr (MetaVirtualMonitor) second_virtual_monitor = NULL;
+  MetaBackend *backend = meta_context_get_backend (test_context);
+  ClutterActor *stage = meta_backend_get_stage (backend);
+  MetaWaylandTestClient *wayland_test_client;
+  MetaWindow *window;
+  MtkRectangle frame_rect, monitor_layout;
+  g_autofree char *session_id = NULL;
+  MetaWindowWayland *wl_window;
+  uint32_t state_change_serial = 0U;
+
+  monitor_layout = get_primary_logical_monitor_layout ();
+
+  second_virtual_monitor = meta_create_test_monitor (test_context,
+                                                     800, 600, 60.0);
+
+  session_manager = meta_context_get_session_manager (test_context);
+  g_signal_connect (session_manager, "session-instantiated",
+                    G_CALLBACK (on_session_instantiated), &session_id);
+
+  /* Launch client once, resize window */
+  wayland_test_client =
+    meta_wayland_test_client_new (test_context, "xdg-session-management-restore");
+
+  wait_for_sync_point (0);
+  wait_for_paint (stage);
+
+  window = find_client_window ("toplevel1");
+  g_assert_nonnull (window);
+
+  /* Move to second monitor */
+  meta_window_move_resize_frame (window, FALSE,
+                                 monitor_layout.width + 123, 123, 100, 100);
+  wait_for_paint (stage);
+
+  frame_rect = meta_window_config_get_rect (window->config);
+  g_assert_cmpint (frame_rect.x, ==, monitor_layout.width + 123);
+  g_assert_cmpint (frame_rect.y, ==, 123);
+  g_assert_cmpint (frame_rect.width, ==, 100);
+  g_assert_cmpint (frame_rect.height, ==, 100);
+
+  meta_window_tile (window, META_TILE_LEFT);
+
+  wl_window = META_WINDOW_WAYLAND (window);
+  meta_window_wayland_get_pending_serial (wl_window, &state_change_serial);
+  g_assert_cmpint (state_change_serial, !=, 0U);
+  while (meta_window_wayland_peek_configuration (wl_window, state_change_serial))
+    g_main_context_iteration (NULL, TRUE);
+
+  frame_rect = meta_window_config_get_rect (window->config);
+  g_assert_cmpint (frame_rect.x, ==, 640);
+  g_assert_cmpint (frame_rect.y, ==, 0);
+  g_assert_cmpint (frame_rect.width, ==, 400);
+  g_assert_cmpint (frame_rect.height, ==, 600);
+
+  meta_wayland_test_driver_emit_sync_event (test_driver, 0);
+  meta_wayland_test_client_finish (wayland_test_client);
+
+  g_assert_nonnull (session_id);
+
+  /* Launch client again, check window persists left-tiled on then
+   * second monitor */
+  wayland_test_client =
+    meta_wayland_test_client_new_with_args (test_context,
+                                            "xdg-session-management-restore",
+                                            session_id,
+                                            NULL);
+  wait_for_sync_point (0);
+
+  window = find_client_window ("toplevel1");
+  g_assert_nonnull (window);
+  g_assert_nonnull (window->monitor);
+  frame_rect = meta_window_config_get_rect (window->config);
+  g_assert_cmpint (frame_rect.x, ==, 640);
+  g_assert_cmpint (frame_rect.y, ==, 0);
+  g_assert_cmpint (frame_rect.width, ==, 400);
+  g_assert_cmpint (frame_rect.height, ==, 600);
+
+  g_signal_handlers_disconnect_by_func (session_manager, on_session_instantiated,
+                                        &session_id);
+
+  meta_wayland_test_driver_emit_sync_event (test_driver, 0);
+  meta_wayland_test_client_finish (wayland_test_client);
+}
+
 static void
 toplevel_sessions_restore_fullscreen_monitor_removed (void)
 {
@@ -1365,11 +1542,17 @@ init_tests (void)
   g_test_add_func ("/wayland/toplevel/sessions-restore",
                    toplevel_sessions_restore);
 #ifdef MUTTER_PRIVILEGED_TEST
+  (void)(toplevel_sessions_restore_maximized);
+  (void)(toplevel_sessions_restore_tiled);
   (void)(toplevel_sessions_restore_fullscreen);
   (void)(toplevel_sessions_restore_fullscreen_monitor_removed);
   (void)(toplevel_bounds_struts);
   (void)(toplevel_bounds_monitors);
 #else
+  g_test_add_func ("/wayland/toplevel/sessions-restore-maximized",
+                   toplevel_sessions_restore_maximized);
+  g_test_add_func ("/wayland/toplevel/sessions-restore-tiled",
+                   toplevel_sessions_restore_tiled);
   g_test_add_func ("/wayland/toplevel/sessions-restore-fullscreen",
                    toplevel_sessions_restore_fullscreen);
   g_test_add_func ("/wayland/toplevel/sessions-restore-fullscreen-monitor-removed",
