@@ -316,7 +316,8 @@ meta_wayland_actor_surface_apply_state (MetaWaylandSurfaceRole  *surface_role,
   MetaWaylandActorSurfacePrivate *priv =
     meta_wayland_actor_surface_get_instance_private (actor_surface);
 
-  if (priv->actor && !wl_list_empty (&pending->frame_callback_list))
+  if (priv->actor &&
+      (!wl_list_empty (&pending->frame_callback_list) || pending->fifo_wait))
     meta_surface_actor_schedule_update (priv->actor);
 
   meta_wayland_actor_surface_queue_frame_callbacks (actor_surface, pending);
