@@ -13,14 +13,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_XWAYLAND_SURFACE_H
-#define META_XWAYLAND_SURFACE_H
+#pragma once
 
 #include "meta/types.h"
 #include "wayland/meta-wayland-actor-surface.h"
@@ -34,5 +31,3 @@ G_DECLARE_FINAL_TYPE (MetaXwaylandSurface,
 void
 meta_xwayland_surface_associate_with_window (MetaXwaylandSurface *xwayland_surface,
                                              MetaWindow          *window);
-
-#endif /* META_XWAYLAND_SURFACE_H */
