@@ -39,13 +39,14 @@
 #include "core/meta-workspace-manager-private.h"
 #include "core/window-private.h"
 #include "core/workspace-private.h"
-#include "meta/group.h"
 #include "meta/meta-backend.h"
 #include "meta/meta-context.h"
 #include "mtk/mtk-x11.h"
 #include "x11/meta-startup-notification-x11.h"
 #include "x11/meta-x11-display-private.h"
 #include "x11/meta-x11-event-source.h"
+#include "x11/meta-x11-frame.h"
+#include "x11/meta-x11-group-private.h"
 #include "x11/meta-x11-selection-private.h"
 #include "x11/meta-x11-selection-input-stream-private.h"
 #include "x11/meta-x11-selection-output-stream-private.h"
@@ -803,15 +804,18 @@ handle_window_focus_event (MetaX11Display *x11_display,
   MetaWindow *focus_window;
 #ifdef WITH_VERBOSE_MODE
   const char *window_type;
+  MetaFrame *frame;
 
   /* Note the event can be on either the window or the frame,
    * we focus the frame for output-only windows
    */
   if (window)
     {
+      frame = meta_window_x11_get_frame (window);
+
       if (event->event == meta_window_x11_get_xwindow (window))
         window_type = "client window";
-      else if (window->frame && event->event == window->frame->xwindow)
+      else if (frame && event->event == frame->xwindow)
         window_type = "frame window";
       else
         window_type = "unknown client window";
@@ -972,8 +976,8 @@ handle_input_xevent (MetaX11Display *x11_display,
           meta_display_handle_window_enter (display,
                                             window,
                                             enter_event->time,
-                                            enter_event->root_x,
-                                            enter_event->root_y);
+                                            (int) enter_event->root_x,
+                                            (int) enter_event->root_y);
         }
       break;
     case XI_Leave:
@@ -1108,8 +1112,10 @@ process_selection_request (MetaX11Display *x11_display,
                           event->xselectionrequest.selection);
       mtk_x11_error_trap_pop (x11_display->xdisplay);
 
-      meta_verbose ("Selection request with selection %s window 0x%lx not a WM_Sn selection we recognize",
-                    str ? str : "(bad atom)", event->xselectionrequest.owner);
+      meta_topic (META_DEBUG_X11,
+                  "Selection request with selection %s window 0x%lx "
+                  "not a WM_Sn selection we recognize",
+                  str ? str : "(bad atom)", event->xselectionrequest.owner);
 
       meta_XFree (str);
 
@@ -1188,10 +1194,10 @@ process_selection_request (MetaX11Display *x11_display,
               event->xselectionrequest.requestor,
               False, 0L, (XEvent*)&reply);
 
-  meta_verbose ("Handled selection request");
+  meta_topic (META_DEBUG_X11, "Handled selection request");
 }
 
-static gboolean
+static void
 close_display_idle_cb (gpointer user_data)
 {
   MetaX11Display *x11_display = META_X11_DISPLAY (user_data);
@@ -1202,8 +1208,6 @@ close_display_idle_cb (gpointer user_data)
                       x11_display->xselectionclear_timestamp);
   x11_display->display_close_idle = 0;
   meta_context_terminate (context);
-
-  return G_SOURCE_REMOVE;
 }
 
 static gboolean
@@ -1220,22 +1224,24 @@ process_selection_clear (MetaX11Display *x11_display,
                           event->xselectionclear.selection);
       mtk_x11_error_trap_pop (x11_display->xdisplay);
 
-      meta_verbose ("Selection clear with selection %s window 0x%lx not a WM_Sn selection we recognize",
-                    str ? str : "(bad atom)", event->xselectionclear.window);
+      meta_topic (META_DEBUG_X11,
+                  "Selection clear with selection %s window 0x%lx "
+                  "not a WM_Sn selection we recognize",
+                  str ? str : "(bad atom)", event->xselectionclear.window);
 
       meta_XFree (str);
 
       return FALSE;
     }
 
-  meta_verbose ("Got selection clear for on display %s",
-                x11_display->name);
+  meta_topic (META_DEBUG_X11, "Got selection clear for on display %s",
+              x11_display->name);
 
   /* We can't close a Display in an event handler. */
   if (!x11_display->display_close_idle)
     {
       x11_display->xselectionclear_timestamp = event->xselectionclear.time;
-      x11_display->display_close_idle = g_idle_add (close_display_idle_cb, x11_display);
+      x11_display->display_close_idle = g_idle_add_once (close_display_idle_cb, x11_display);
     }
 
   return TRUE;
@@ -1248,12 +1254,16 @@ notify_bell (MetaX11Display *x11_display,
   MetaDisplay *display = x11_display->display;
   XkbBellNotifyEvent *xkb_bell_event = (XkbBellNotifyEvent*) xkb_ev;
   MetaWindow *window;
+  MetaFrame *frame;
 
   window = meta_x11_display_lookup_x_window (x11_display,
                                              xkb_bell_event->window);
-  if (!window && display->focus_window && display->focus_window->frame)
-    window = display->focus_window;
-
+  if (!window && display->focus_window)
+    {
+      frame = meta_window_x11_get_frame (display->focus_window);
+      if (frame)
+        window = display->focus_window;
+    }
   x11_display->last_bell_time = xkb_ev->time;
   if (!meta_bell_notify (display, window) &&
       meta_prefs_bell_is_audible ())
@@ -1277,10 +1287,13 @@ handle_other_xevent (MetaX11Display *x11_display,
   MetaWindow *window;
   MetaWindow *property_for_window;
   gboolean frame_was_receiver;
+  MetaFrame *frame = NULL;
 
   modified = event_get_modified_window (x11_display, event);
   window = modified != None ? meta_x11_display_lookup_x_window (x11_display, modified) : NULL;
-  frame_was_receiver = (window && window->frame && modified == window->frame->xwindow);
+  if (window)
+    frame = meta_window_x11_get_frame (window);
+  frame_was_receiver = frame && modified == frame->xwindow;
 
   /* We only want to respond to _NET_WM_USER_TIME property notify
    * events on _NET_WM_USER_TIME_WINDOW windows; in particular,
@@ -1341,7 +1354,7 @@ handle_other_xevent (MetaX11Display *x11_display,
         }
       else
         {
-          meta_topic (META_DEBUG_SHAPES,
+          meta_topic (META_DEBUG_X11,
                       "ShapeNotify not on a client window (window 0x%lx)",
                       modified);
         }
@@ -1387,7 +1400,7 @@ handle_other_xevent (MetaX11Display *x11_display,
           if (frame_was_receiver)
             {
               mtk_x11_error_trap_push (x11_display->xdisplay);
-              meta_window_destroy_frame (window->frame->window);
+              meta_window_destroy_frame (frame->window);
               mtk_x11_error_trap_pop (x11_display->xdisplay);
             }
           else
@@ -1439,6 +1452,8 @@ handle_other_xevent (MetaX11Display *x11_display,
         {
           window = meta_window_x11_new (display, event->xmap.window,
                                         FALSE, META_COMP_EFFECT_CREATE);
+          if (window)
+            meta_window_x11_configure (window);
         }
       else if (window && window->restore_focus_on_map &&
                window->reparents_pending == 0)
@@ -1482,7 +1497,7 @@ handle_other_xevent (MetaX11Display *x11_display,
               window = meta_x11_display_lookup_x_window (x11_display,
                                                          client_window);
 
-              if (window != NULL && window->decorated && !window->frame)
+              if (window != NULL && window->decorated && !meta_window_x11_is_ssd (window))
                 {
                   meta_window_x11_set_frame_xwindow (window,
                                                      event->xmaprequest.window);
@@ -1495,19 +1510,24 @@ handle_other_xevent (MetaX11Display *x11_display,
 
           window = meta_window_x11_new (display, event->xmaprequest.window,
                                         FALSE, META_COMP_EFFECT_CREATE);
+          if (window)
+            meta_window_x11_configure (window);
         }
       else
         {
-          meta_verbose ("MapRequest on %s mapped = %d minimized = %d",
-                        window->desc, window->mapped, window->minimized);
+          meta_topic (META_DEBUG_X11,
+                      "MapRequest on %s mapped = %d minimized = %d",
+                      window->desc, window->mapped, window->minimized);
 
           if (window->minimized && !frame_was_receiver)
             {
               meta_window_unminimize (window);
               if (window->workspace != workspace_manager->active_workspace)
                 {
-                  meta_verbose ("Changing workspace due to MapRequest mapped = %d minimized = %d",
-                                window->mapped, window->minimized);
+                  meta_topic (META_DEBUG_X11,
+                              "Changing workspace due to MapRequest "
+                              "mapped = %d minimized = %d",
+                              window->mapped, window->minimized);
                   meta_window_change_workspace (window,
                                                 workspace_manager->active_workspace);
                 }
@@ -1558,8 +1578,10 @@ handle_other_xevent (MetaX11Display *x11_display,
           xwc.height = event->xconfigurerequest.height;
           xwc.border_width = event->xconfigurerequest.border_width;
 
-          meta_verbose ("Configuring withdrawn window to %d,%d %dx%d border %d (some values may not be in mask)",
-                        xwc.x, xwc.y, xwc.width, xwc.height, xwc.border_width);
+          meta_topic (META_DEBUG_X11,
+                      "Configuring withdrawn window to %d,%d %dx%d border %d "
+                      "(some values may not be in mask)",
+                      xwc.x, xwc.y, xwc.width, xwc.height, xwc.border_width);
           mtk_x11_error_trap_push (x11_display->xdisplay);
           XConfigureWindow (x11_display->xdisplay, event->xconfigurerequest.window,
                             xwcm, &xwc);
@@ -1569,9 +1591,9 @@ handle_other_xevent (MetaX11Display *x11_display,
         {
           meta_window_x11_configure_request (window, event);
         }
-      else if (frame_was_receiver && window->frame)
+      else if (frame_was_receiver && frame)
         {
-          meta_frame_handle_xevent (window->frame, event);
+          meta_frame_handle_xevent (frame, event);
         }
       break;
     case GravityNotify:
@@ -1591,7 +1613,7 @@ handle_other_xevent (MetaX11Display *x11_display,
         else if (property_for_window && !frame_was_receiver)
           meta_window_x11_property_notify (property_for_window, event);
         else if (frame_was_receiver)
-          meta_frame_handle_xevent (window->frame, event);
+          meta_frame_handle_xevent (frame, event);
 
         group = meta_x11_display_lookup_group (x11_display,
                                                event->xproperty.window);
@@ -1651,9 +1673,10 @@ handle_other_xevent (MetaX11Display *x11_display,
                   space = event->xclient.data.l[0];
                   time = event->xclient.data.l[1];
 
-                  meta_verbose ("Request to change current workspace to %d with "
-                                "specified timestamp of %u",
-                                space, time);
+                  meta_topic (META_DEBUG_X11,
+                              "Request to change current workspace to %d with "
+                              "specified timestamp of %u",
+                              space, time);
 
                   workspace = meta_workspace_manager_get_workspace_by_index (workspace_manager, space);
 
@@ -1667,7 +1690,7 @@ handle_other_xevent (MetaX11Display *x11_display,
                     }
                   else
                     {
-                      meta_verbose ("Don't know about workspace %d", space);
+                      meta_topic (META_DEBUG_X11, "Don't know about workspace %d", space);
                     }
                 }
               else if (event->xclient.message_type ==
@@ -1677,8 +1700,8 @@ handle_other_xevent (MetaX11Display *x11_display,
 
                   num_spaces = event->xclient.data.l[0];
 
-                  meta_verbose ("Request to set number of workspaces to %d",
-                                num_spaces);
+                  meta_topic (META_DEBUG_X11, "Request to set number of workspaces to %d",
+                              num_spaces);
 
                   meta_prefs_set_num_workspaces (num_spaces);
                 }
@@ -1691,8 +1714,8 @@ handle_other_xevent (MetaX11Display *x11_display,
                   showing_desktop = event->xclient.data.l[0] != 0;
                   /* FIXME: Braindead protocol doesn't have a timestamp */
                   timestamp = meta_x11_display_get_current_time_roundtrip (x11_display);
-                  meta_verbose ("Request to %s desktop",
-                                showing_desktop ? "show" : "hide");
+                  meta_topic (META_DEBUG_X11, "Request to %s desktop",
+                              showing_desktop ? "show" : "hide");
 
                   if (showing_desktop)
                     meta_workspace_manager_show_desktop (workspace_manager, timestamp);
@@ -1705,7 +1728,7 @@ handle_other_xevent (MetaX11Display *x11_display,
               else if (event->xclient.message_type ==
                        x11_display->atom_WM_PROTOCOLS)
                 {
-                  meta_verbose ("Received WM_PROTOCOLS message");
+                  meta_topic (META_DEBUG_X11, "Received WM_PROTOCOLS message");
 
                   if ((Atom)event->xclient.data.l[0] == x11_display->atom__NET_WM_PING)
                     {
@@ -1719,7 +1742,8 @@ handle_other_xevent (MetaX11Display *x11_display,
           if (event->xclient.message_type ==
               x11_display->atom__NET_REQUEST_FRAME_EXTENTS)
             {
-              meta_verbose ("Received _NET_REQUEST_FRAME_EXTENTS message");
+              meta_topic (META_DEBUG_X11,
+                          "Received _NET_REQUEST_FRAME_EXTENTS message");
               process_request_frame_extents (x11_display, event);
             }
         }
@@ -1780,10 +1804,13 @@ static gboolean
 window_has_xwindow (MetaWindow *window,
                     Window      xwindow)
 {
+  MetaFrame *frame;
+
   if (meta_window_x11_get_xwindow (window) == xwindow)
     return TRUE;
 
-  if (window->frame && window->frame->xwindow == xwindow)
+  frame = meta_window_x11_get_frame (window);
+  if (frame && frame->xwindow == xwindow)
     return TRUE;
 
   return FALSE;
@@ -1836,11 +1863,12 @@ meta_x11_display_handle_xevent (MetaX11Display *x11_display,
 {
   MetaDisplay *display = x11_display->display;
   MetaContext *context = meta_display_get_context (display);
-  MetaBackend *backend = meta_context_get_backend (context);
-  Window modified;
-  gboolean bypass_compositor = FALSE;
-  XIEvent *input_event;
+#ifdef HAVE_X11
   MetaCursorTracker *cursor_tracker;
+  MetaBackend *backend = meta_context_get_backend (context);
+#endif
+  gboolean bypass_compositor G_GNUC_UNUSED = FALSE;
+  XIEvent *input_event;
 #ifdef HAVE_XWAYLAND
   MetaWaylandCompositor *wayland_compositor;
 #endif
@@ -1883,8 +1911,10 @@ meta_x11_display_handle_xevent (MetaX11Display *x11_display,
 
   display->current_time = event_get_time (x11_display, event);
 
+#ifdef HAVE_X11
   if (META_IS_BACKEND_X11 (backend))
     meta_backend_x11_reset_cached_logical_monitor (META_BACKEND_X11 (backend));
+#endif
 
   if (x11_display->focused_by_us &&
       event->xany.serial > x11_display->focus_serial &&
@@ -1901,6 +1931,7 @@ meta_x11_display_handle_xevent (MetaX11Display *x11_display,
       x11_display->is_server_focus = FALSE;
     }
 
+#ifdef HAVE_X11
   if (event->xany.window == x11_display->xroot)
     {
       cursor_tracker = meta_backend_get_cursor_tracker (backend);
@@ -1916,8 +1947,7 @@ meta_x11_display_handle_xevent (MetaX11Display *x11_display,
             }
         }
     }
-
-  modified = event_get_modified_window (x11_display, event);
+#endif
 
   input_event = get_input_event (x11_display, event);
 
@@ -1936,11 +1966,15 @@ meta_x11_display_handle_xevent (MetaX11Display *x11_display,
     }
 
  out:
+#ifdef HAVE_X11
   if (!bypass_compositor && META_IS_COMPOSITOR_X11 (display->compositor))
     {
       MetaCompositorX11 *compositor_x11 =
         META_COMPOSITOR_X11 (display->compositor);
       MetaWindow *window;
+      Window modified;
+
+      modified = event_get_modified_window (x11_display, event);
 
       if (modified != None)
         window = meta_x11_display_lookup_x_window (x11_display, modified);
@@ -1949,6 +1983,7 @@ meta_x11_display_handle_xevent (MetaX11Display *x11_display,
 
       meta_compositor_x11_process_xevent (compositor_x11, event, window);
     }
+#endif /* HAVE_X11 */
 
   display->current_time = META_CURRENT_TIME;
 
